@@ -33,10 +33,12 @@ import htsjdk.samtools.reference.ReferenceSequence;
 import htsjdk.samtools.util.CollectionUtil;
 import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.Log;
-import picard.cmdline.CommandLineProgramProperties;
-import picard.cmdline.Option;
+import org.broadinstitute.barclay.argparser.Argument;
+import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
+import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.cmdline.StandardOptionDefinitions;
-import picard.cmdline.programgroups.Metrics;
+import picard.cmdline.argumentcollections.ReferenceArgumentCollection;
+import picard.cmdline.programgroups.DiagnosticsAndQCProgramGroup;
 
 import java.io.File;
 import java.util.EnumSet;
@@ -73,10 +75,11 @@ import java.util.Set;
  * @author Doug Voet (dvoet at broadinstitute dot org)
  */
 @CommandLineProgramProperties(
-        usage = CollectAlignmentSummaryMetrics.USAGE_SUMMARY + CollectAlignmentSummaryMetrics.USAGE_DETAILS,
-        usageShort = CollectAlignmentSummaryMetrics.USAGE_SUMMARY,
-        programGroup = Metrics.class
+        summary = CollectAlignmentSummaryMetrics.USAGE_SUMMARY + CollectAlignmentSummaryMetrics.USAGE_DETAILS,
+        oneLineSummary = CollectAlignmentSummaryMetrics.USAGE_SUMMARY,
+        programGroup = DiagnosticsAndQCProgramGroup.class
 )
+@DocumentedFeature
 public class CollectAlignmentSummaryMetrics extends SinglePassSamProgram {
     static final String USAGE_SUMMARY = "<b>Produces a summary of alignment metrics from a SAM or BAM file.</b>  ";
     static final String USAGE_DETAILS = "This tool takes a SAM/BAM file input and produces metrics detailing the quality of the read " +
@@ -101,25 +104,21 @@ public class CollectAlignmentSummaryMetrics extends SinglePassSamProgram {
 
     private static final Log log = Log.getInstance(CollectAlignmentSummaryMetrics.class);
 
-    @Option(doc="Paired-end reads above this insert size will be considered chimeric along with inter-chromosomal pairs.")
+    @Argument(doc="Paired-end reads above this insert size will be considered chimeric along with inter-chromosomal pairs.")
     public int MAX_INSERT_SIZE = ChimeraUtil.DEFAULT_INSERT_SIZE_LIMIT;
 
-    @Option(doc="Paired-end reads that do not have this expected orientation will be considered chimeric.")
+    @Argument(doc="Paired-end reads that do not have this expected orientation will be considered chimeric.")
     public Set<PairOrientation> EXPECTED_PAIR_ORIENTATIONS = EnumSet.copyOf(ChimeraUtil.DEFAULT_EXPECTED_ORIENTATIONS);
 
-    @Option(doc="List of adapter sequences to use when processing the alignment metrics.")
+    @Argument(doc="List of adapter sequences to use when processing the alignment metrics.")
 	public List<String> ADAPTER_SEQUENCE = AdapterUtility.DEFAULT_ADAPTER_SEQUENCE;
 
-    @Option(shortName="LEVEL", doc="The level(s) at which to accumulate metrics.")
+    @Argument(shortName="LEVEL", doc="The level(s) at which to accumulate metrics.")
     public Set<MetricAccumulationLevel> METRIC_ACCUMULATION_LEVEL = CollectionUtil.makeSet(MetricAccumulationLevel.ALL_READS);
 
-    @Option(shortName="BS", doc="Whether the SAM or BAM file consists of bisulfite sequenced reads.")
+    @Argument(shortName="BS", doc="Whether the SAM or BAM file consists of bisulfite sequenced reads.")
     public boolean IS_BISULFITE_SEQUENCED = false;
 
-    //overridden to make it visible on the commandline and to change the doc.
-    @Option(shortName = StandardOptionDefinitions.REFERENCE_SHORT_NAME, doc = "Reference sequence file. Note that while this argument isn't required, without it only a small subset of the metrics will be calculated. Note also that if a reference sequence is provided, it must be accompanied by a sequence dictionary.",  optional = true, overridable = true)
-    public File REFERENCE_SEQUENCE = Defaults.REFERENCE_FASTA;
-
     private AlignmentSummaryMetricsCollector collector;
 
     /** Required main method implementation. */
@@ -155,4 +154,23 @@ public class CollectAlignmentSummaryMetrics extends SinglePassSamProgram {
 
         file.write(OUTPUT);
     }
+
+    //overridden to make it visible on the commandline and to change the doc.
+    @Override
+    protected ReferenceArgumentCollection makeReferenceArgumentCollection() {
+        return new CollectAlignmentRefArgCollection();
+    }
+
+    public static class CollectAlignmentRefArgCollection implements ReferenceArgumentCollection {
+        @Argument(shortName = StandardOptionDefinitions.REFERENCE_SHORT_NAME,
+                doc = "Reference sequence file. Note that while this argument isn't required, without it only a small subset of the metrics will be calculated. Note also that if a reference sequence is provided, it must be accompanied by a sequence dictionary.",
+                optional = true)
+        public File REFERENCE_SEQUENCE = Defaults.REFERENCE_FASTA;
+
+        @Override
+        public File getReferenceFile() {
+            return REFERENCE_SEQUENCE;
+        };
+    }
+
 }
