@@ -8,15 +8,16 @@ import htsjdk.samtools.filter.*;
 import htsjdk.samtools.metrics.MetricsFile;
 import htsjdk.samtools.reference.ReferenceSequence;
 import htsjdk.samtools.util.*;
+import org.broadinstitute.barclay.argparser.Argument;
+import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
 import picard.analysis.SinglePassSamProgram;
 import picard.analysis.artifacts.SequencingArtifactMetrics.BaitBiasDetailMetrics;
 import picard.analysis.artifacts.SequencingArtifactMetrics.BaitBiasSummaryMetrics;
 import picard.analysis.artifacts.SequencingArtifactMetrics.PreAdapterDetailMetrics;
 import picard.analysis.artifacts.SequencingArtifactMetrics.PreAdapterSummaryMetrics;
-import picard.cmdline.CommandLineProgramProperties;
-import picard.cmdline.Option;
-import picard.cmdline.programgroups.Metrics;
+import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
+import picard.cmdline.programgroups.DiagnosticsAndQCProgramGroup;
 import picard.util.DbSnpBitSetUtil;
 import picard.util.VariantType;
 
@@ -44,10 +45,11 @@ import static picard.cmdline.StandardOptionDefinitions.MINIMUM_MAPPING_QUALITY_S
  *
  */
 @CommandLineProgramProperties(
-        usage = CollectSequencingArtifactMetrics.USAGE_SUMMARY + CollectSequencingArtifactMetrics.USAGE_DETAILS,
-        usageShort = CollectSequencingArtifactMetrics.USAGE_SUMMARY,
-        programGroup = Metrics.class
+        summary = CollectSequencingArtifactMetrics.USAGE_SUMMARY + CollectSequencingArtifactMetrics.USAGE_DETAILS,
+        oneLineSummary = CollectSequencingArtifactMetrics.USAGE_SUMMARY,
+        programGroup = DiagnosticsAndQCProgramGroup.class
 )
+@DocumentedFeature
 public class CollectSequencingArtifactMetrics extends SinglePassSamProgram {
 static final String USAGE_SUMMARY = "Collect metrics to quantify single-base sequencing artifacts.  ";
 static final String USAGE_DETAILS = "<p>This tool examines two sources of sequencing errors associated with hybrid selection "+
@@ -87,49 +89,49 @@ static final String USAGE_DETAILS = "<p>This tool examines two sources of sequen
 "for complete descriptions of the output metrics produced by this tool. "+
 "<hr />"
 ;
-    @Option(doc = "An optional list of intervals to restrict analysis to.", optional = true)
+    @Argument(doc = "An optional list of intervals to restrict analysis to.", optional = true)
     public File INTERVALS;
 
-    @Option(doc = "VCF format dbSNP file, used to exclude regions around known polymorphisms from analysis.", optional = true)
+    @Argument(doc = "VCF format dbSNP file, used to exclude regions around known polymorphisms from analysis.", optional = true)
     public File DB_SNP;
 
-    @Option(shortName = "Q", doc = "The minimum base quality score for a base to be included in analysis.")
+    @Argument(shortName = "Q", doc = "The minimum base quality score for a base to be included in analysis.")
     public int MINIMUM_QUALITY_SCORE = 20;
 
-    @Option(shortName = MINIMUM_MAPPING_QUALITY_SHORT_NAME, doc = "The minimum mapping quality score for a base to be included in analysis.")
+    @Argument(shortName = MINIMUM_MAPPING_QUALITY_SHORT_NAME, doc = "The minimum mapping quality score for a base to be included in analysis.")
     public int MINIMUM_MAPPING_QUALITY = 30;
 
-    @Option(shortName = "MIN_INS", doc = "The minimum insert size for a read to be included in analysis.")
+    @Argument(shortName = "MIN_INS", doc = "The minimum insert size for a read to be included in analysis.")
     public int MINIMUM_INSERT_SIZE = 60;
 
-    @Option(shortName = "MAX_INS", doc = "The maximum insert size for a read to be included in analysis. Set to 0 to have no maximum.")
+    @Argument(shortName = "MAX_INS", doc = "The maximum insert size for a read to be included in analysis. Set to 0 to have no maximum.")
     public int MAXIMUM_INSERT_SIZE = 600;
 
-    @Option(shortName = "UNPAIRED", doc = "Include unpaired reads. If set to true then all paired reads will be included as well - " +
+    @Argument(shortName = "UNPAIRED", doc = "Include unpaired reads. If set to true then all paired reads will be included as well - " +
             "MINIMUM_INSERT_SIZE and MAXIMUM_INSERT_SIZE will be ignored.")
     public boolean INCLUDE_UNPAIRED = false;
 
-    @Option(shortName = "DUPES", doc = "Include duplicate reads. If set to true then all reads flagged as duplicates will be included as well.")
+    @Argument(shortName = "DUPES", doc = "Include duplicate reads. If set to true then all reads flagged as duplicates will be included as well.")
     public boolean INCLUDE_DUPLICATES = false;
 
-    @Option(shortName = "NON_PF", doc = "Whether or not to include non-PF reads.")
+    @Argument(shortName = "NON_PF", doc = "Whether or not to include non-PF reads.")
     public boolean INCLUDE_NON_PF_READS = false;
 
-    @Option(shortName = "TANDEM", doc = "Set to true if mate pairs are being sequenced from the same strand, " +
+    @Argument(shortName = "TANDEM", doc = "Set to true if mate pairs are being sequenced from the same strand, " +
             "i.e. they're expected to face the same direction.")
     public boolean TANDEM_READS = false;
 
-    @Option(doc = "When available, use original quality scores for filtering.")
+    @Argument(doc = "When available, use original quality scores for filtering.")
     public boolean USE_OQ = true;
 
-    @Option(doc = "The number of context bases to include on each side of the assayed base.")
+    @Argument(doc = "The number of context bases to include on each side of the assayed base.")
     public int CONTEXT_SIZE = 1;
 
-    @Option(doc = "If specified, only print results for these contexts in the detail metrics output. " +
-                  "However, the summary metrics output will still take all contexts into consideration.")
-    public Set<String> CONTEXTS_TO_PRINT = new HashSet<String>();
+    @Argument(doc = "If specified, only print results for these contexts in the detail metrics output. " +
+                  "However, the summary metrics output will still take all contexts into consideration.", optional = true)
+    public Set<String> CONTEXTS_TO_PRINT = new HashSet<>();
 
-    @Option(shortName = "EXT", doc="Append the given file extension to all metric file names (ex. OUTPUT.pre_adapter_summary_metrics.EXT). None if null", optional=true)
+    @Argument(shortName = "EXT", doc="Append the given file extension to all metric file names (ex. OUTPUT.pre_adapter_summary_metrics.EXT). None if null", optional=true)
     public String FILE_EXTENSION = null;
 
     private static final String UNKNOWN_LIBRARY = "UnknownLibrary";
@@ -148,15 +150,20 @@ static final String USAGE_DETAILS = "<p>This tool examines two sources of sequen
     private String currentRefString = null;
     private int currentRefIndex = -1;
 
-    private final Set<String> samples = new HashSet<String>();
-    private final Set<String> libraries = new HashSet<String>();
-    private final Map<String, ArtifactCounter> artifactCounters = new HashMap<String, ArtifactCounter>();
+    private final Set<String> samples = new HashSet<>();
+    private final Set<String> libraries = new HashSet<>();
+    private final Map<String, ArtifactCounter> artifactCounters = new HashMap<>();
 
     private static final Log log = Log.getInstance(CollectSequencingArtifactMetrics.class);
 
+    @Override
+    protected boolean requiresReference() {
+        return true;
+    }
+
     @Override
     protected String[] customCommandLineValidation() {
-        final List<String> messages = new ArrayList<String>();
+        final List<String> messages = new ArrayList<>();
 
         final int contextFullLength = 2 * CONTEXT_SIZE + 1;
         if (CONTEXT_SIZE < 0) messages.add("CONTEXT_SIZE cannot be negative");
@@ -172,8 +179,6 @@ static final String USAGE_DETAILS = "<p>This tool examines two sources of sequen
             messages.add("MAXIMUM_INSERT_SIZE cannot be less than MINIMUM_INSERT_SIZE unless set to 0");
         }
 
-        if (REFERENCE_SEQUENCE == null) messages.add("REFERENCE_SEQUENCE must be provided.");
-
         return messages.isEmpty() ? null : messages.toArray(new String[messages.size()]);
     }
 
@@ -209,7 +214,7 @@ static final String USAGE_DETAILS = "<p>This tool examines two sources of sequen
         }
 
         // set record-level filters
-        final List<SamRecordFilter> filters = new ArrayList<SamRecordFilter>();
+        final List<SamRecordFilter> filters = new ArrayList<>();
         if (!INCLUDE_NON_PF_READS) filters.add(new FailsVendorReadQualityFilter());
         filters.add(new NotPrimaryAlignmentFilter());
         if (!INCLUDE_DUPLICATES) filters.add(new DuplicateReadFilter());
@@ -222,7 +227,7 @@ static final String USAGE_DETAILS = "<p>This tool examines two sources of sequen
         recordFilter = new AggregateFilter(filters);
 
         // set up the artifact counters
-        final String sampleAlias = StringUtil.join(",", new ArrayList<String>(samples));
+        final String sampleAlias = StringUtil.join(",", new ArrayList<>(samples));
         for (final String library : libraries) {
             artifactCounters.put(library, new ArtifactCounter(sampleAlias, library, CONTEXT_SIZE, TANDEM_READS));
         }
