@@ -24,28 +24,24 @@
 
 package picard.illumina;
 
-import htsjdk.samtools.BAMRecordCodec;
-import htsjdk.samtools.SAMFileHeader;
-import htsjdk.samtools.SAMFileWriter;
-import htsjdk.samtools.SAMFileWriterFactory;
-import htsjdk.samtools.SAMReadGroupRecord;
-import htsjdk.samtools.SAMRecord;
-import htsjdk.samtools.SAMRecordQueryNameComparator;
+import htsjdk.samtools.*;
 import htsjdk.samtools.util.CollectionUtil;
 import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.Iso8601Date;
 import htsjdk.samtools.util.Log;
 import htsjdk.samtools.util.SortingCollection;
 import htsjdk.samtools.util.StringUtil;
+import org.broadinstitute.barclay.argparser.Argument;
+import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
+import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
 import picard.cmdline.CommandLineProgram;
-import picard.cmdline.CommandLineProgramProperties;
-import picard.cmdline.Option;
-import picard.cmdline.programgroups.Illumina;
 import picard.cmdline.StandardOptionDefinitions;
+import picard.cmdline.programgroups.BaseCallingProgramGroup;
+import picard.illumina.parser.IlluminaFileUtil;
 import picard.illumina.parser.ReadStructure;
-import picard.illumina.parser.ReadType;
 import picard.illumina.parser.readers.BclQualityEvaluationStrategy;
+import picard.util.AdapterPair;
 import picard.util.IlluminaUtil;
 import picard.util.IlluminaUtil.IlluminaAdapterPair;
 import picard.util.TabbedTextFileWithHeaderParser;
@@ -55,7 +51,6 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collections;
 import java.util.Comparator;
 import java.util.Date;
 import java.util.HashMap;
@@ -81,7 +76,7 @@ import java.util.Set;
  * reading Illumina data for their respective tiles from disk, and as they collect that data, it is fed back into the
  * TileReadAggregator.  When a TileReader completes a tile, it notifies the TileReadAggregator, which reviews what was
  * read and conditionally queues its writing to disk, baring in mind the requirements of write-order described in the
- * previous paragraph.  As writes complete, the TileReadAggregator re-evalutes the state of reads/writes and may queue
+ * previous paragraph.  As writes complete, the TileReadAggregator re-evaluates the state of reads/writes and may queue
  * more writes.  When all barcodes for all tiles have been written, the TileReadAggregator shuts down.
  * <p/>
  * The TileReadAggregator controls task execution using a specialized ThreadPoolExecutor.  It accepts special Runnables
@@ -95,17 +90,18 @@ import java.util.Set;
  * @author mccowan@broadinstitute.org
  */
 @CommandLineProgramProperties(
-        usage = IlluminaBasecallsToSam.USAGE_SUMMARY + IlluminaBasecallsToSam.USAGE_DETAILS,
-        usageShort = IlluminaBasecallsToSam.USAGE_SUMMARY,
-        programGroup = Illumina.class
+        summary = IlluminaBasecallsToSam.USAGE_SUMMARY + IlluminaBasecallsToSam.USAGE_DETAILS,
+        oneLineSummary = IlluminaBasecallsToSam.USAGE_SUMMARY,
+        programGroup = BaseCallingProgramGroup.class
 )
+@DocumentedFeature
 public class IlluminaBasecallsToSam extends CommandLineProgram {
-    static final String USAGE_SUMMARY = "Transforms raw Illumina sequencing data into an unmapped SAM or BAM file."  ;
+    static final String USAGE_SUMMARY = "Transforms raw Illumina sequencing data into an unmapped SAM or BAM file.";
     static final String USAGE_DETAILS = "<p>The IlluminaBaseCallsToSam program collects, demultiplexes, and sorts reads across all " +
             "of the tiles of a lane via barcode to produce an unmapped SAM/BAM file.  An unmapped BAM file is often referred to as a uBAM.  " +
             "All barcode, sample, and library data is provided in the LIBRARY_PARAMS file.  Note, this LIBRARY_PARAMS file " +
             "should be formatted according to the specifications indicated below.  The following is an example of a properly" +
-            " formmated LIBRARY_PARAMS file:</p>" +
+            " formatted LIBRARY_PARAMS file:</p>" +
             "BARCODE_1\tOUTPUT\tSAMPLE_ALIAS\tLIBRARY_NAME\n" +
             "AAAAAAAA\tSA_AAAAAAAA.bam\tSA_AAAAAAAA\tLN_AAAAAAAA\n" +
             "AAAAGAAG\tSA_AAAAGAAG.bam\tSA_AAAAGAAG\tLN_AAAAGAAG\n" +
@@ -127,67 +123,69 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
             "      IGNORE_UNEXPECTED_BARCODES=true \\<br />" +
             "      LIBRARY_PARAMS=library.params " +
             "</pre>" +
-            "<hr />"
-    ;
+            "<hr />";
 
 
     // The following attributes define the command-line arguments
 
     public static final String USAGE = "Generate a SAM or BAM file from data in an Illumina basecalls output directory";
 
-    @Option(doc = "The basecalls directory. ", shortName = "B")
+    @Argument(doc = "The basecalls directory. ", shortName = "B")
     public File BASECALLS_DIR;
-    
-    @Option(doc = "The barcodes directory with _barcode.txt files (generated by ExtractIlluminaBarcodes). If not set, use BASECALLS_DIR. ", shortName = "BCD", optional = true)
+
+    @Argument(doc = "The barcodes directory with _barcode.txt files (generated by ExtractIlluminaBarcodes). If not set, use BASECALLS_DIR. ", shortName = "BCD", optional = true)
     public File BARCODES_DIR;
 
-    @Option(doc = "Lane number. ", shortName = StandardOptionDefinitions.LANE_SHORT_NAME)
+    @Argument(doc = "Lane number. ", shortName = StandardOptionDefinitions.LANE_SHORT_NAME)
     public Integer LANE;
 
-    @Option(doc = "Deprecated (use LIBRARY_PARAMS).  The output SAM or BAM file. Format is determined by extension.",
+    @Argument(doc = "Deprecated (use LIBRARY_PARAMS).  The output SAM or BAM file. Format is determined by extension.",
             shortName = StandardOptionDefinitions.OUTPUT_SHORT_NAME,
             mutex = {"BARCODE_PARAMS", "LIBRARY_PARAMS"})
     public File OUTPUT;
 
-    @Option(doc = "The barcode of the run.  Prefixed to read names.")
+    @Argument(doc = "The barcode of the run.  Prefixed to read names.")
     public String RUN_BARCODE;
 
-    @Option(doc = "Deprecated (use LIBRARY_PARAMS).  The name of the sequenced sample",
+    @Argument(doc = "Deprecated (use LIBRARY_PARAMS).  The name of the sequenced sample",
             shortName = StandardOptionDefinitions.SAMPLE_ALIAS_SHORT_NAME,
             mutex = {"BARCODE_PARAMS", "LIBRARY_PARAMS"})
     public String SAMPLE_ALIAS;
 
-    @Option(doc = "ID used to link RG header record with RG tag in SAM record.  " +
+    @Argument(doc = "ID used to link RG header record with RG tag in SAM record.  " +
             "If these are unique in SAM files that get merged, merge performance is better.  " +
             "If not specified, READ_GROUP_ID will be set to <first 5 chars of RUN_BARCODE>.<LANE> .",
             shortName = StandardOptionDefinitions.READ_GROUP_ID_SHORT_NAME, optional = true)
     public String READ_GROUP_ID;
 
-    @Option(doc = "Deprecated (use LIBRARY_PARAMS).  The name of the sequenced library",
+    @Argument(doc = "Deprecated (use LIBRARY_PARAMS).  The name of the sequenced library",
             shortName = StandardOptionDefinitions.LIBRARY_NAME_SHORT_NAME,
             optional = true,
             mutex = {"BARCODE_PARAMS", "LIBRARY_PARAMS"})
     public String LIBRARY_NAME;
 
-    @Option(doc = "The name of the sequencing center that produced the reads.  Used to set the RG.CN tag.", optional = true)
-    public String SEQUENCING_CENTER = "BI";
+    @Argument(doc = "The name of the sequencing center that produced the reads.  Used to set the @RG->CN header tag.")
+    public String SEQUENCING_CENTER;
 
-    @Option(doc = "The start date of the run.", optional = true)
+    @Argument(doc = "The start date of the run.", optional = true)
     public Date RUN_START_DATE;
 
-    @Option(doc = "The name of the sequencing technology that produced the read.", optional = true)
+    @Argument(doc = "The name of the sequencing technology that produced the read.", optional = true)
     public String PLATFORM = "illumina";
 
-    @Option(doc = ReadStructure.PARAMETER_DOC, shortName = "RS")
+    @Argument(doc = "Whether to include the barcode information in the @RG->BC header tag. Defaults to false until included in the SAM spec.")
+    public boolean INCLUDE_BC_IN_RG_TAG = false;
+
+    @Argument(doc = ReadStructure.PARAMETER_DOC, shortName = "RS")
     public String READ_STRUCTURE;
 
-    @Option(doc = "Deprecated (use LIBRARY_PARAMS).  Tab-separated file for creating all output BAMs for barcoded run " +
+    @Argument(doc = "Deprecated (use LIBRARY_PARAMS).  Tab-separated file for creating all output BAMs for barcoded run " +
             "with single IlluminaBasecallsToSam invocation.  Columns are BARCODE, OUTPUT, SAMPLE_ALIAS, and " +
             "LIBRARY_NAME.  Row with BARCODE=N is used to specify a file for no barcode match",
             mutex = {"OUTPUT", "SAMPLE_ALIAS", "LIBRARY_NAME", "LIBRARY_PARAMS"})
     public File BARCODE_PARAMS;
 
-    @Option(doc = "Tab-separated file for creating all output BAMs for a lane with single IlluminaBasecallsToSam " +
+    @Argument(doc = "Tab-separated file for creating all output BAMs for a lane with single IlluminaBasecallsToSam " +
             "invocation.  The columns are OUTPUT, SAMPLE_ALIAS, and LIBRARY_NAME, BARCODE_1, BARCODE_2 ... BARCODE_X " +
             "where X = number of barcodes per cluster (optional).  Row with BARCODE_1 set to 'N' is used to specify a file " +
             "for no barcode match.  You may also provide any 2 letter RG header attributes (excluding PU, CN, PL, and" +
@@ -196,64 +194,81 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
             mutex = {"OUTPUT", "SAMPLE_ALIAS", "LIBRARY_NAME", "BARCODE_PARAMS"})
     public File LIBRARY_PARAMS;
 
-    @Option(doc = "Which adapters to look for in the read.")
-    public List<IlluminaAdapterPair> ADAPTERS_TO_CHECK = new ArrayList<IlluminaAdapterPair>(
+    @Argument(doc = "Which adapters to look for in the read.")
+    public List<IlluminaAdapterPair> ADAPTERS_TO_CHECK = new ArrayList<>(
             Arrays.asList(IlluminaAdapterPair.INDEXED,
                     IlluminaAdapterPair.DUAL_INDEXED,
                     IlluminaAdapterPair.NEXTERA_V2,
                     IlluminaAdapterPair.FLUIDIGM));
 
-    @Option(doc = "The number of threads to run in parallel. If NUM_PROCESSORS = 0, number of cores is automatically set to " +
+    @Argument(doc = "For specifying adapters other than standard Illumina", optional = true)
+    public String FIVE_PRIME_ADAPTER;
+
+    @Argument(doc = "For specifying adapters other than standard Illumina", optional = true)
+    public String THREE_PRIME_ADAPTER;
+
+    @Argument(doc = "The number of threads to run in parallel. If NUM_PROCESSORS = 0, number of cores is automatically set to " +
             "the number of cores available on the machine. If NUM_PROCESSORS < 0, then the number of cores used will" +
             " be the number available on the machine less NUM_PROCESSORS.")
     public Integer NUM_PROCESSORS = 0;
 
-    @Option(doc = "If set, this is the first tile to be processed (used for debugging).  Note that tiles are not processed" +
+    @Argument(doc = "If set, this is the first tile to be processed (used for debugging).  Note that tiles are not processed" +
             " in numerical order.",
+            mutex = "PROCESS_SINGLE_TILE",
             optional = true)
     public Integer FIRST_TILE;
 
-    @Option(doc = "If set, process no more than this many tiles (used for debugging).", optional = true)
+    @Argument(doc = "If set, process no more than this many tiles (used for debugging).", optional = true)
     public Integer TILE_LIMIT;
 
-    @Option(doc = "If true, call System.gc() periodically.  This is useful in cases in which the -Xmx value passed " +
+    @Argument(doc = "If set, process only the tile number given and append the tile number to the output file name.",
+            mutex = "FIRST_TILE",
+            optional = true)
+    public Integer PROCESS_SINGLE_TILE;
+
+    @Argument(doc = "If true, call System.gc() periodically.  This is useful in cases in which the -Xmx value passed " +
             "is larger than the available memory.")
     public Boolean FORCE_GC = true;
 
-    @Option(doc="Apply EAMSS filtering to identify inappropriately quality scored bases towards the ends of reads" +
+    @Argument(doc = "Apply EAMSS filtering to identify inappropriately quality scored bases towards the ends of reads" +
             " and convert their quality scores to Q2.")
     public boolean APPLY_EAMSS_FILTER = true;
 
-    @Option(doc = "Configure SortingCollections to store this many records before spilling to disk. For an indexed" +
+    @Argument(doc = "Configure SortingCollections to store this many records before spilling to disk. For an indexed" +
             " run, each SortingCollection gets this value/number of indices.")
     public int MAX_READS_IN_RAM_PER_TILE = 1200000;
 
-    @Option(doc="The minimum quality (after transforming 0s to 1s) expected from reads.  If qualities are lower than this value, an error is thrown." +
+    @Argument(doc = "The minimum quality (after transforming 0s to 1s) expected from reads.  If qualities are lower than this value, an error is thrown." +
             "The default of 2 is what the Illumina's spec describes as the minimum, but in practice the value has been observed lower.")
     public int MINIMUM_QUALITY = BclQualityEvaluationStrategy.ILLUMINA_ALLEGED_MINIMUM_QUALITY;
 
-    @Option(doc="Whether to include non-PF reads", shortName="NONPF", optional=true)
+    @Argument(doc = "Whether to include non-PF reads", shortName = "NONPF", optional = true)
     public boolean INCLUDE_NON_PF_READS = true;
 
-    @Option(doc="Whether to ignore reads whose barcodes are not found in LIBRARY_PARAMS.  Useful when outputting " +
-            "BAMs for only a subset of the barcodes in a lane.", shortName="IGNORE_UNEXPECTED")
+    @Argument(doc = "Whether to ignore reads whose barcodes are not found in LIBRARY_PARAMS.  Useful when outputting " +
+            "BAMs for only a subset of the barcodes in a lane.", shortName = "IGNORE_UNEXPECTED")
     public boolean IGNORE_UNEXPECTED_BARCODES = false;
 
-    @Option(doc="The tag to use to store any molecular indexes.  If more than one molecular index is found, they will be concatenated and stored here.", optional=true)
+    @Argument(doc = "The tag to use to store any molecular indexes.  If more than one molecular index is found, they will be concatenated and stored here.", optional = true)
     public String MOLECULAR_INDEX_TAG = "RX";
 
-    @Option(doc="The tag to use to store any molecular index base qualities.  If more than one molecular index is found, their qualities will be concatenated and stored here " +
-            "(.i.e. the number of \"M\" operators in the READ_STRUCTURE)", optional=true)
+    @Argument(doc = "The tag to use to store any molecular index base qualities.  If more than one molecular index is found, their qualities will be concatenated and stored here " +
+            "(.i.e. the number of \"M\" operators in the READ_STRUCTURE)", optional = true)
     public String MOLECULAR_INDEX_BASE_QUALITY_TAG = "QX";
 
-    @Option(doc="The list of tags to store each molecular index.  The number of tags should match the number of molecular indexes.", optional=true)
+    @Argument(doc = "The list of tags to store each molecular index.  The number of tags should match the number of molecular indexes.", optional = true)
     public List<String> TAG_PER_MOLECULAR_INDEX;
 
-    private final Map<String, SAMFileWriterWrapper> barcodeSamWriterMap = new HashMap<String, SAMFileWriterWrapper>();
+    @Argument(doc = "When should the sample barcode (as read by the sequencer) be placed on the reads in the BC tag?")
+    public ClusterDataToSamConverter.PopulateBarcode BARCODE_POPULATION_STRATEGY = ClusterDataToSamConverter.PopulateBarcode.ORPHANS_ONLY;
+
+    @Argument(doc = "Should the barcode quality be included when the sample barcode is included?")
+    public boolean INCLUDE_BARCODE_QUALITY = false;
+
+    private final Map<String, SAMFileWriterWrapper> barcodeSamWriterMap = new HashMap<>();
     private ReadStructure readStructure;
-    IlluminaBasecallsConverter<SAMRecordsForCluster> basecallsConverter;
+    private BasecallsConverter<SAMRecordsForCluster> basecallsConverter;
     private static final Log log = Log.getInstance(IlluminaBasecallsToSam.class);
-    private BclQualityEvaluationStrategy bclQualityEvaluationStrategy;
 
     @Override
     protected int doWork() {
@@ -266,7 +281,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
      * Prepares loggers, initiates garbage collection thread, parses arguments and initialized variables appropriately/
      */
     private void initialize() {
-        this.bclQualityEvaluationStrategy = new BclQualityEvaluationStrategy(MINIMUM_QUALITY);
+        final BclQualityEvaluationStrategy bclQualityEvaluationStrategy = new BclQualityEvaluationStrategy(MINIMUM_QUALITY);
 
         if (OUTPUT != null) {
             IOUtil.assertFileIsWritable(OUTPUT);
@@ -277,31 +292,44 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
         }
 
         if (OUTPUT != null) {
-            barcodeSamWriterMap.put(null, buildSamFileWriter(OUTPUT, SAMPLE_ALIAS, LIBRARY_NAME, buildSamHeaderParameters(null)));
+            barcodeSamWriterMap.put(null, buildSamFileWriter(OUTPUT, SAMPLE_ALIAS, LIBRARY_NAME, buildSamHeaderParameters(null), true));
         } else {
             populateWritersFromLibraryParams();
         }
 
         final int numOutputRecords = readStructure.templates.length();
+        // Combine any adapters and custom adapter pairs from the command line into an array for use in clipping
+        final List<AdapterPair> adapters = new ArrayList<>(ADAPTERS_TO_CHECK);
 
-        basecallsConverter = new IlluminaBasecallsConverter<SAMRecordsForCluster>(BASECALLS_DIR, BARCODES_DIR, LANE, readStructure,
-                barcodeSamWriterMap, true, MAX_READS_IN_RAM_PER_TILE/numOutputRecords, TMP_DIR, NUM_PROCESSORS, FORCE_GC,
-                FIRST_TILE, TILE_LIMIT, new QueryNameComparator(), new Codec(numOutputRecords), SAMRecordsForCluster.class,
-                bclQualityEvaluationStrategy, this.APPLY_EAMSS_FILTER, INCLUDE_NON_PF_READS, IGNORE_UNEXPECTED_BARCODES);
+        if (FIVE_PRIME_ADAPTER != null && THREE_PRIME_ADAPTER != null) {
+            adapters.add(new CustomAdapterPair(FIVE_PRIME_ADAPTER, THREE_PRIME_ADAPTER));
+        }
 
-        log.info("DONE_READING STRUCTURE IS " + readStructure.toString());
-
-        /**
+        if (IlluminaFileUtil.hasCbcls(BASECALLS_DIR, LANE)) {
+            if (BARCODES_DIR == null) BARCODES_DIR = BASECALLS_DIR;
+            basecallsConverter = new NewIlluminaBasecallsConverter<>(BASECALLS_DIR, BARCODES_DIR, LANE, readStructure,
+                    barcodeSamWriterMap, true, Math.max(1, MAX_READS_IN_RAM_PER_TILE / numOutputRecords),
+                    TMP_DIR, NUM_PROCESSORS,
+                    FIRST_TILE, TILE_LIMIT, new QueryNameComparator(),
+                    new Codec(numOutputRecords),
+                    SAMRecordsForCluster.class, bclQualityEvaluationStrategy, IGNORE_UNEXPECTED_BARCODES);
+        } else {
+            basecallsConverter = new IlluminaBasecallsConverter<>(BASECALLS_DIR, BARCODES_DIR, LANE, readStructure,
+                    barcodeSamWriterMap, true, MAX_READS_IN_RAM_PER_TILE / numOutputRecords, TMP_DIR, NUM_PROCESSORS, FORCE_GC,
+                    FIRST_TILE, TILE_LIMIT, new QueryNameComparator(), new Codec(numOutputRecords), SAMRecordsForCluster.class,
+                    bclQualityEvaluationStrategy, APPLY_EAMSS_FILTER, INCLUDE_NON_PF_READS, IGNORE_UNEXPECTED_BARCODES);
+        }
+        /*
          * Be sure to pass the outputReadStructure to ClusterDataToSamConverter, which reflects the structure of the output cluster
          * data which may be different from the input read structure (specifically if there are skips).
          */
         final ClusterDataToSamConverter converter = new ClusterDataToSamConverter(RUN_BARCODE, READ_GROUP_ID,
-                basecallsConverter.getFactory().getOutputReadStructure(), ADAPTERS_TO_CHECK)
+                basecallsConverter.getFactory().getOutputReadStructure(), adapters, BARCODE_POPULATION_STRATEGY, INCLUDE_BARCODE_QUALITY )
                 .withMolecularIndexTag(MOLECULAR_INDEX_TAG)
                 .withMolecularIndexQualityTag(MOLECULAR_INDEX_BASE_QUALITY_TAG)
                 .withTagPerMolecularIndex(TAG_PER_MOLECULAR_INDEX);
         basecallsConverter.setConverter(converter);
-
+        log.info("DONE_READING STRUCTURE IS " + readStructure.toString());
     }
 
     /**
@@ -312,17 +340,17 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
      * @return actualCols - expectedCols
      */
     private Set<String> findAndFilterExpectedColumns(final Set<String> actualCols, final Set<String> expectedCols) {
-        final Set<String> missingColumns = new HashSet<String>(expectedCols);
+        final Set<String> missingColumns = new HashSet<>(expectedCols);
         missingColumns.removeAll(actualCols);
 
         if (!missingColumns.isEmpty()) {
             throw new PicardException(String.format(
                     "LIBRARY_PARAMS file %s is missing the following columns: %s.",
                     LIBRARY_PARAMS.getAbsolutePath(), StringUtil.join(", ", missingColumns
-            )));
+                    )));
         }
 
-        final Set<String> remainingColumns = new HashSet<String>(actualCols);
+        final Set<String> remainingColumns = new HashSet<>(actualCols);
         remainingColumns.removeAll(expectedCols);
         return remainingColumns;
     }
@@ -357,7 +385,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
         final TabbedTextFileWithHeaderParser libraryParamsParser = new TabbedTextFileWithHeaderParser(LIBRARY_PARAMS);
 
         final Set<String> expectedColumnLabels = CollectionUtil.makeSet("OUTPUT", "SAMPLE_ALIAS", "LIBRARY_NAME");
-        final List<String> barcodeColumnLabels = new ArrayList<String>();
+        final List<String> barcodeColumnLabels = new ArrayList<>();
         if (readStructure.sampleBarcodes.length() == 1) {
             //For the single barcode read case, the barcode label name can either by BARCODE or BARCODE_1
             if (libraryParamsParser.hasColumn("BARCODE")) {
@@ -381,7 +409,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
             List<String> barcodeValues = null;
 
             if (!barcodeColumnLabels.isEmpty()) {
-                barcodeValues = new ArrayList<String>();
+                barcodeValues = new ArrayList<>();
                 for (final String barcodeLabel : barcodeColumnLabels) {
                     barcodeValues.add(row.getField(barcodeLabel));
                 }
@@ -399,8 +427,17 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
                 samHeaderParams.put(tagName, row.getField(tagName));
             }
 
-            final SAMFileWriterWrapper writer = buildSamFileWriter(new File(row.getField("OUTPUT")),
-                    row.getField("SAMPLE_ALIAS"), row.getField("LIBRARY_NAME"), samHeaderParams);
+            File outputFile = new File(row.getField("OUTPUT"));
+
+            // If we are processing a single tile we want to append the tile number to the output file name. This is
+            // done to avoid file overwrites if you are running tile based processing in parallel.
+            if (PROCESS_SINGLE_TILE != null) {
+                outputFile = new File(outputFile.getParentFile(),
+                        PROCESS_SINGLE_TILE + "." + outputFile.getName());
+            }
+
+            final SAMFileWriterWrapper writer = buildSamFileWriter(outputFile,
+                    row.getField("SAMPLE_ALIAS"), row.getField("LIBRARY_NAME"), samHeaderParams, true);
             barcodeSamWriterMap.put(key, writer);
         }
         if (barcodeSamWriterMap.isEmpty()) {
@@ -418,15 +455,26 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
      * @return A Map of ReadGroupHeaderTags -> Values
      */
     private Map<String, String> buildSamHeaderParameters(final List<String> barcodes) {
-        final Map<String, String> params = new LinkedHashMap<String, String>();
+        final Map<String, String> params = new LinkedHashMap<>();
 
         String platformUnit = RUN_BARCODE + "." + LANE;
-        if (barcodes != null) platformUnit += ("." + IlluminaUtil.barcodeSeqsToString(barcodes));
+        if (barcodes != null) {
+            final String barcodeString = IlluminaUtil.barcodeSeqsToString(barcodes);
+            platformUnit += "." + barcodeString;
+            if (INCLUDE_BC_IN_RG_TAG) {
+                params.put("BC", barcodeString);
+            }
+        }
 
-        params.put("PL", PLATFORM);
-        params.put("PU", platformUnit);
-        params.put("CN", SEQUENCING_CENTER);
-        params.put("DT", RUN_START_DATE == null ? null : new Iso8601Date(RUN_START_DATE).toString());
+        if (PLATFORM != null) {
+            params.put(SAMReadGroupRecord.PLATFORM_TAG, PLATFORM);
+        }
+
+        params.put(SAMReadGroupRecord.PLATFORM_UNIT_TAG, platformUnit);
+        if (SEQUENCING_CENTER != null) {
+            params.put(SAMReadGroupRecord.SEQUENCING_CENTER_TAG, SEQUENCING_CENTER);
+        }
+        params.put(SAMReadGroupRecord.DATE_RUN_PRODUCED_TAG, RUN_START_DATE == null ? null : new Iso8601Date(RUN_START_DATE).toString());
 
         return params;
     }
@@ -441,7 +489,8 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
      * @return A SAMFileWriter
      */
     private SAMFileWriterWrapper buildSamFileWriter(final File output, final String sampleAlias,
-                                                    final String libraryName, final Map<String, String> headerParameters) {
+                                                    final String libraryName, final Map<String, String> headerParameters,
+                                                    final boolean presorted) {
         IOUtil.assertFileIsWritable(output);
         final SAMReadGroupRecord rg = new SAMReadGroupRecord(READ_GROUP_ID);
         rg.setSample(sampleAlias);
@@ -457,7 +506,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
 
         header.setSortOrder(SAMFileHeader.SortOrder.queryname);
         header.addReadGroup(rg);
-        return new SAMFileWriterWrapper(new SAMFileWriterFactory().makeSAMOrBAMWriter(header, true, output));
+        return new SAMFileWriterWrapper(new SAMFileWriterFactory().makeSAMOrBAMWriter(header, presorted, output));
     }
 
     public static void main(final String[] args) {
@@ -470,7 +519,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
      * Any options set by command-line parser can be validated.
      *
      * @return null if command line is valid.  If command line is invalid, returns an array of error message
-     *         to be written to the appropriate place.
+     * to be written to the appropriate place.
      */
     @Override
     protected String[] customCommandLineValidation() {
@@ -478,7 +527,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
             LIBRARY_PARAMS = BARCODE_PARAMS;
         }
 
-        final ArrayList<String> messages = new ArrayList<String>();
+        final ArrayList<String> messages = new ArrayList<>();
 
         readStructure = new ReadStructure(READ_STRUCTURE);
         if (!readStructure.sampleBarcodes.isEmpty() && LIBRARY_PARAMS == null) {
@@ -494,6 +543,17 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
             messages.add("The number of tags given in TAG_PER_MOLECULAR_INDEX does not match the number of molecular indexes in READ_STRUCTURE");
         }
 
+        if ((FIVE_PRIME_ADAPTER == null) != (THREE_PRIME_ADAPTER == null)) {
+            messages.add("THREE_PRIME_ADAPTER and FIVE_PRIME_ADAPTER must either both be null or both be set.");
+        }
+
+        // If we are processing a single tile we need to set TILE_LIMIT and FIRST_TILE for the underlying
+        // basecalls converter.
+        if (PROCESS_SINGLE_TILE != null) {
+            TILE_LIMIT = 1;
+            FIRST_TILE = PROCESS_SINGLE_TILE;
+        }
+
         if (messages.isEmpty()) {
             return null;
         }
@@ -501,7 +561,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
     }
 
     private static final class SAMFileWriterWrapper
-            implements IlluminaBasecallsConverter.ConvertedClusterDataWriter<SAMRecordsForCluster> {
+            implements BasecallsConverter.ConvertedClusterDataWriter<SAMRecordsForCluster> {
         public final SAMFileWriter writer;
 
         private SAMFileWriterWrapper(final SAMFileWriter writer) {
@@ -531,6 +591,7 @@ public class IlluminaBasecallsToSam extends CommandLineProgram {
 
     static class QueryNameComparator implements Comparator<SAMRecordsForCluster> {
         private final SAMRecordQueryNameComparator comparator = new SAMRecordQueryNameComparator();
+
         @Override
         public int compare(final SAMRecordsForCluster s1, final SAMRecordsForCluster s2) {
             return comparator.compare(s1.records[0], s2.records[0]);
