@@ -23,44 +23,16 @@
  */
 package picard.sam;
 
-import htsjdk.samtools.BAMRecordCodec;
-import htsjdk.samtools.Cigar;
-import htsjdk.samtools.CigarElement;
-import htsjdk.samtools.CigarOperator;
-import htsjdk.samtools.ReservedTagConstants;
-import htsjdk.samtools.SAMFileHeader;
-import htsjdk.samtools.SAMFileHeader.SortOrder;
-import htsjdk.samtools.SAMFileWriter;
-import htsjdk.samtools.SAMFileWriterFactory;
-import htsjdk.samtools.SAMProgramRecord;
-import htsjdk.samtools.SAMRecord;
-import htsjdk.samtools.SAMRecordCoordinateComparator;
-import htsjdk.samtools.SAMRecordQueryNameComparator;
-import htsjdk.samtools.SAMSequenceDictionary;
-import htsjdk.samtools.SAMSequenceRecord;
-import htsjdk.samtools.SAMTag;
-import htsjdk.samtools.SAMUtils;
-import htsjdk.samtools.SamPairUtil;
-import htsjdk.samtools.SamReader;
-import htsjdk.samtools.SamReaderFactory;
+import htsjdk.samtools.*;
 import htsjdk.samtools.filter.FilteringSamIterator;
 import htsjdk.samtools.filter.SamRecordFilter;
 import htsjdk.samtools.reference.ReferenceSequenceFileWalker;
-import htsjdk.samtools.util.CigarUtil;
-import htsjdk.samtools.util.CloseableIterator;
-import htsjdk.samtools.util.CloserUtil;
-import htsjdk.samtools.util.IOUtil;
-import htsjdk.samtools.util.Log;
-import htsjdk.samtools.util.ProgressLogger;
-import htsjdk.samtools.util.SequenceUtil;
-import htsjdk.samtools.util.SortingCollection;
+import htsjdk.samtools.SAMFileHeader.SortOrder;
+import htsjdk.samtools.util.*;
 import picard.PicardException;
 
 import java.io.File;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.Iterator;
-import java.util.List;
+import java.util.*;
 
 /**
  * Abstract class that coordinates the general task of taking in a set of alignment information,
@@ -88,21 +60,23 @@ public abstract class AbstractAlignmentMerger {
     public static final int MAX_RECORDS_IN_RAM = 500000;
 
     private static final char[] RESERVED_ATTRIBUTE_STARTS = {'X', 'Y', 'Z'};
+    private int crossSpeciesReads = 0;
 
     private final Log log = Log.getInstance(AbstractAlignmentMerger.class);
     private final ProgressLogger progress = new ProgressLogger(this.log, 1000000, "Merged", "records");
 
     private final File unmappedBamFile;
     private final File targetBamFile;
-    private final SAMSequenceDictionary sequenceDictionary;
     private ReferenceSequenceFileWalker refSeq = null;
     private final boolean clipAdapters;
     private final boolean bisulfiteSequence;
     private SAMProgramRecord programRecord;
     private final boolean alignedReadsOnly;
     private final SAMFileHeader header;
-    private final List<String> attributesToRetain = new ArrayList<String>();
-    private final List<String> attributesToRemove = new ArrayList<String>();
+    private final List<String> attributesToRetain = new ArrayList<>();
+    private final List<String> attributesToRemove = new ArrayList<>();
+    private Set<String> attributesToReverse = new TreeSet<>(SAMRecord.TAGS_TO_REVERSE);
+    private Set<String> attributesToReverseComplement = new TreeSet<>(SAMRecord.TAGS_TO_REVERSE_COMPLEMENT);
     protected final File referenceFasta;
     private final Integer read1BasesTrimmed;
     private final Integer read2BasesTrimmed;
@@ -115,6 +89,9 @@ public abstract class AbstractAlignmentMerger {
     private boolean keepAlignerProperPairFlags = false;
     private boolean addMateCigar = false;
     private boolean unmapContaminantReads = false;
+    private UnmappingReadStrategy unmappingReadsStrategy = UnmappingReadStrategy.DO_NOT_CHANGE;
+    private boolean addPGTagToReads = true;
+
 
     private final SamRecordFilter alignmentFilter = new SamRecordFilter() {
         public boolean filterOut(final SAMRecord record) {
@@ -147,17 +124,50 @@ public abstract class AbstractAlignmentMerger {
 
         /** Adds a record to the sink. */
         void add(final SAMRecord rec) {
-            if (writer != null) writer.addAlignment(rec);
-            if (sorter != null) sorter.add(rec);
+            if (writer != null) {
+                writer.addAlignment(rec);
+            }
+            if (sorter != null) {
+                sorter.add(rec);
+            }
         }
 
         /** Closes the underlying resource. */
         void close() {
-            if (this.writer != null) this.writer.close();
-            if (this.sorter != null) this.sorter.doneAdding();
+            if (this.writer != null) {
+                this.writer.close();
+            }
+            if (this.sorter != null) {
+                this.sorter.doneAdding();
+            }
         }
     }
 
+    public enum UnmappingReadStrategy {
+        // Leave on record, and copy to tag
+        COPY_TO_TAG(false, true),
+        // Leave on record, but do not create additional tag
+        DO_NOT_CHANGE(false, false),
+        // Add tag with information, and remove from standard fields in record
+        MOVE_TO_TAG(true, true);
+
+        private final boolean resetMappingInformation, populatePATag;
+
+        UnmappingReadStrategy(final boolean resetMappingInformation, final boolean populatePATag) {
+            this.resetMappingInformation = resetMappingInformation;
+            this.populatePATag = populatePATag;
+        }
+
+        public boolean isResetMappingInformation() {
+            return resetMappingInformation;
+        }
+
+        public boolean isPopulatePaTag() {
+            return populatePATag;
+        }
+    }
+
+    protected abstract SAMSequenceDictionary getDictionaryForMergedBam();
 
     protected abstract CloseableIterator<SAMRecord> getQuerynameSortedAlignedRecords();
 
@@ -165,6 +175,43 @@ public abstract class AbstractAlignmentMerger {
 
     protected boolean isContaminant(final HitsForInsert hits) { return false; } // default implementation
 
+    /** constructor with a default setting for unmappingReadsStrategy.
+     *
+     * see full constructor for parameters
+     *
+     *
+     */
+    public AbstractAlignmentMerger(final File unmappedBamFile, final File targetBamFile,
+                                   final File referenceFasta, final boolean clipAdapters,
+                                   final boolean bisulfiteSequence, final boolean alignedReadsOnly,
+                                   final SAMProgramRecord programRecord, final List<String> attributesToRetain,
+                                   final List<String> attributesToRemove,
+                                   final Integer read1BasesTrimmed, final Integer read2BasesTrimmed,
+                                   final List<SamPairUtil.PairOrientation> expectedOrientations,
+                                   final SortOrder sortOrder,
+                                   final PrimaryAlignmentSelectionStrategy primaryAlignmentSelectionStrategy,
+                                   final boolean addMateCigar,
+                                   final boolean unmapContaminantReads) {
+        this(unmappedBamFile,
+             targetBamFile,
+             referenceFasta,
+             clipAdapters,
+             bisulfiteSequence,
+             alignedReadsOnly,
+             programRecord,
+             attributesToRetain,
+             attributesToRemove,
+             read1BasesTrimmed,
+             read2BasesTrimmed,
+             expectedOrientations,
+             sortOrder,
+             primaryAlignmentSelectionStrategy,
+             addMateCigar,
+             unmapContaminantReads,
+             UnmappingReadStrategy.DO_NOT_CHANGE);
+    }
+
+
     /**
      * Constructor
      *
@@ -194,8 +241,10 @@ public abstract class AbstractAlignmentMerger {
      * @param primaryAlignmentSelectionStrategy What to do when there are multiple primary alignments, or multiple
      *                                          alignments but none primary, for a read or read pair.
      * @param addMateCigar                      True if we are to add or maintain the mate CIGAR (MC) tag, false if we are to remove or not include.
-     * @param unmapContaminantReads             If true, identify reads having the signature of contamination from a foreign organism (i.e. mostly clipped bases),
+     * @param unmapContaminantReads             If true, identify reads having the signature of cross-species contamination (i.e. mostly clipped bases),
      *                                          and mark them as unmapped.
+     * @param unmappingReadsStrategy            An enum describing how to deal with reads whose mapping information are being removed (currently this happens due to cross-species
+     *                                          contamination). Ignored unless unmapContaminantReads is true.
      */
     public AbstractAlignmentMerger(final File unmappedBamFile, final File targetBamFile,
                                    final File referenceFasta, final boolean clipAdapters,
@@ -204,10 +253,11 @@ public abstract class AbstractAlignmentMerger {
                                    final List<String> attributesToRemove,
                                    final Integer read1BasesTrimmed, final Integer read2BasesTrimmed,
                                    final List<SamPairUtil.PairOrientation> expectedOrientations,
-                                   final SAMFileHeader.SortOrder sortOrder,
+                                   final SortOrder sortOrder,
                                    final PrimaryAlignmentSelectionStrategy primaryAlignmentSelectionStrategy,
                                    final boolean addMateCigar,
-                                   final boolean unmapContaminantReads) {
+                                   final boolean unmapContaminantReads,
+                                   final UnmappingReadStrategy unmappingReadsStrategy) {
         IOUtil.assertFileIsReadable(unmappedBamFile);
         IOUtil.assertFileIsWritable(targetBamFile);
         IOUtil.assertFileIsReadable(referenceFasta);
@@ -217,11 +267,6 @@ public abstract class AbstractAlignmentMerger {
         this.referenceFasta = referenceFasta;
 
         this.refSeq = new ReferenceSequenceFileWalker(referenceFasta);
-        this.sequenceDictionary = refSeq.getSequenceDictionary();
-        if (this.sequenceDictionary == null) {
-            throw new PicardException("No sequence dictionary found for " + referenceFasta.getAbsolutePath() +
-                    ".  Use CreateSequenceDictionary.jar to create a sequence dictionary.");
-        }
 
         this.clipAdapters = clipAdapters;
         this.bisulfiteSequence = bisulfiteSequence;
@@ -233,7 +278,7 @@ public abstract class AbstractAlignmentMerger {
         if (programRecord != null) {
             setProgramRecord(programRecord);
         }
-        header.setSequenceDictionary(this.sequenceDictionary);
+
         if (attributesToRetain != null) {
             this.attributesToRetain.addAll(attributesToRetain);
         }
@@ -241,12 +286,10 @@ public abstract class AbstractAlignmentMerger {
             this.attributesToRemove.addAll(attributesToRemove);
             // attributesToRemove overrides attributesToRetain
             if (!this.attributesToRetain.isEmpty()) {
-                for (String attribute : this.attributesToRemove) {
-                    if (this.attributesToRetain.contains(attribute)) {
-                        log.info("Overriding retaining the " + attribute + " tag since remove overrides retain.");
-                        this.attributesToRetain.remove(attribute);
-                    }
-                }
+                this.attributesToRemove.stream()
+                        .filter(this.attributesToRetain::contains)
+                        .peek(a->log.info("Overriding retaining the " + a + " tag since 'remove' overrides 'retain'."))
+                        .forEach(this.attributesToRetain::remove);
             }
         }
         this.read1BasesTrimmed = read1BasesTrimmed;
@@ -257,6 +300,21 @@ public abstract class AbstractAlignmentMerger {
 
         this.addMateCigar = addMateCigar;
         this.unmapContaminantReads = unmapContaminantReads;
+        this.unmappingReadsStrategy = unmappingReadsStrategy;
+    }
+
+    /** Gets the set of attributes to be reversed on reads marked as negative strand. */
+    public Set<String> getAttributesToReverse() { return attributesToReverse; }
+
+    /** Sets the set of attributes to be reversed on reads marked as negative strand. */
+    public void setAttributesToReverse(Set<String> attributesToReverse) { this.attributesToReverse = attributesToReverse; }
+
+    /** Gets the set of attributes to be reverse complemented on reads marked as negative strand. */
+    public Set<String> getAttributesToReverseComplement() { return attributesToReverseComplement; }
+
+    /** Sets the set of attributes to be reverse complemented on reads marked as negative strand. */
+    public void setAttributesToReverseComplement(Set<String> attributesToReverseComplement) {
+        this.attributesToReverseComplement = attributesToReverseComplement;
     }
 
     /** Allows the caller to override the maximum records in RAM. */
@@ -264,6 +322,14 @@ public abstract class AbstractAlignmentMerger {
         this.maxRecordsInRam = maxRecordsInRam;
     }
 
+    /**
+     * Set addPGTagToReads. If true, the PG will be added to reads when applicable. If false, the PG tag will not be added.
+     * Default is true
+     */
+    public void setAddPGTagToReads(final boolean addPGTagToReads) {
+        this.addPGTagToReads = addPGTagToReads;
+    }
+
     /**
      * Do this unconditionally, not just for aligned records, for two reasons:
      * - An unaligned read has been processed by the aligner, so it is more truthful.
@@ -272,7 +338,7 @@ public abstract class AbstractAlignmentMerger {
      * and a separate chain for the unmapped reads.
      */
     private void maybeSetPgTag(final SAMRecord rec) {
-        if (this.programRecord != null) {
+        if (this.programRecord != null && addPGTagToReads) {
             rec.setAttribute(ReservedTagConstants.PROGRAM_GROUP_ID, this.programRecord.getProgramGroupId());
         }
     }
@@ -292,6 +358,10 @@ public abstract class AbstractAlignmentMerger {
 
         // Get the aligned records and set up the first one
         alignedIterator = new MultiHitAlignedReadIterator(new FilteringSamIterator(getQuerynameSortedAlignedRecords(), alignmentFilter), primaryAlignmentSelectionStrategy);
+
+        // once the aligned Iterator has been set-up we can merge the reference dictionary and that from the input files
+        this.header.setSequenceDictionary(getDictionaryForMergedBam());
+
         HitsForInsert nextAligned = nextAligned();
 
         // Check that the program record we are going to insert is not already used in the unmapped SAM
@@ -312,7 +382,7 @@ public abstract class AbstractAlignmentMerger {
         if (this.sortOrder == SortOrder.coordinate) {
             final SortingCollection<SAMRecord> sorted1 = SortingCollection.newInstance(
                     SAMRecord.class, new BAMRecordCodec(header), new SAMRecordCoordinateComparator(),
-                    MAX_RECORDS_IN_RAM);
+                    this.maxRecordsInRam);
             sink = new Sink(sorted1);
         }
         else { // catches queryname and unsorted
@@ -393,13 +463,19 @@ public abstract class AbstractAlignmentMerger {
                         // this avoids the scenario of having multiple unmapped reads with the same name & pair flags
                         if (!firstToWrite.getReadUnmappedFlag() || isPrimaryAlignment) {
                             addIfNotFiltered(sink, firstToWrite);
-                            if (firstToWrite.getReadUnmappedFlag()) ++unmapped;
-                            else ++aligned;
+                            if (firstToWrite.getReadUnmappedFlag()) {
+                                ++unmapped;
+                            } else {
+                                ++aligned;
+                            }
                         }
                         if (!secondToWrite.getReadUnmappedFlag() || isPrimaryAlignment) {
                             addIfNotFiltered(sink, secondToWrite);
-                            if (!secondToWrite.getReadUnmappedFlag()) ++aligned;
-                            else ++unmapped;
+                            if (!secondToWrite.getReadUnmappedFlag()) {
+                                ++aligned;
+                            } else {
+                                ++unmapped;
+                            }
                         }
                     }
 
@@ -417,7 +493,9 @@ public abstract class AbstractAlignmentMerger {
                             if (!out.getReadUnmappedFlag()) {
                                 addIfNotFiltered(sink, out);
                                 ++aligned;
-                            } else ++unmapped;
+                            } else {
+                                ++unmapped;
+                            }
                         }
                     }
                 } else {
@@ -427,9 +505,14 @@ public abstract class AbstractAlignmentMerger {
                         transferAlignmentInfoToFragment(recToWrite, nextAligned.getFragment(i), unmapDueToContaminant, clone);
                         // Only write unmapped read if it was originally the primary.
                         // this avoids the scenario of having multiple unmapped reads with the same name & pair flags
-                        if (!recToWrite.getReadUnmappedFlag() || isPrimary) addIfNotFiltered(sink, recToWrite);
-                        if (recToWrite.getReadUnmappedFlag()) ++unmapped;
-                        else ++aligned;
+                        if (!recToWrite.getReadUnmappedFlag() || isPrimary) {
+                            addIfNotFiltered(sink, recToWrite);
+                        }
+                        if (recToWrite.getReadUnmappedFlag()) {
+                            ++unmapped;
+                        } else {
+                            ++aligned;
+                        }
                     }
                     // Take all of the supplemental reads which had been stashed and add them (as appropriate) to sorted
                     for (final SAMRecord supplementalRec : nextAligned.getSupplementalFirstOfPairOrFragment()) {
@@ -439,7 +522,9 @@ public abstract class AbstractAlignmentMerger {
                         if (!recToWrite.getReadUnmappedFlag()) {
                             addIfNotFiltered(sink, recToWrite);
                             ++aligned;
-                        } else ++unmapped;
+                        } else {
+                            ++unmapped;
+                        }
                     }
                 }
                 nextAligned = nextAligned();
@@ -500,13 +585,26 @@ public abstract class AbstractAlignmentMerger {
      * No return value, modifies the provided record.
      */
     public static void fixNmMdAndUq(final SAMRecord record, final ReferenceSequenceFileWalker refSeqWalker, final boolean isBisulfiteSequence) {
-        final byte[] referenceBases = refSeqWalker.get(refSeqWalker.getSequenceDictionary().getSequenceIndex(record.getReferenceName())).getBases();
+        final byte[] referenceBases = refSeqWalker.get(record.getReferenceIndex()).getBases();
         // only recalculate NM if it isn't bisulfite, since it needs to be treated specially below
         SequenceUtil.calculateMdAndNmTags(record, referenceBases, true, !isBisulfiteSequence);
         if (isBisulfiteSequence) {  // recalculate the NM tag for bisulfite data
             record.setAttribute(SAMTag.NM.name(), SequenceUtil.calculateSamNmTag(record, referenceBases, 0, isBisulfiteSequence));
         }
+        fixUq(record, refSeqWalker, isBisulfiteSequence);
+    }
+
+    /** Calculates and sets UQ tag from the record and the reference
+     *
+     * @param record the record to be fixed
+     * @param refSeqWalker a ReferenceSequenceWalker that will be used to traverse the reference
+     * @param isBisulfiteSequence a flag indicating whether the sequence came from bisulfite-sequencing.
+     *
+     * No return value, modifies the provided record.
+     */
+    public static void fixUq(final SAMRecord record, final ReferenceSequenceFileWalker refSeqWalker, final boolean isBisulfiteSequence) {
         if (record.getBaseQualities() != SAMRecord.NULL_QUALS) {
+            final byte[] referenceBases = refSeqWalker.get(record.getReferenceIndex()).getBases();
             record.setAttribute(SAMTag.UQ.name(), SequenceUtil.sumQualitiesOfMismatches(record, referenceBases, 0, isBisulfiteSequence));
         }
     }
@@ -517,7 +615,9 @@ public abstract class AbstractAlignmentMerger {
     private void addIfNotFiltered(final Sink out, final SAMRecord rec) {
         if (includeSecondaryAlignments || !rec.getNotPrimaryAlignmentFlag()) {
             out.add(rec);
-            this.progress.record(rec);
+            if (this.progress.record(rec) && crossSpeciesReads > 0) {
+                log.info(String.format("%d Reads have been unmapped due to being suspected of being Cross-species contamination.", crossSpeciesReads));
+            }
         }
     }
 
@@ -543,6 +643,14 @@ public abstract class AbstractAlignmentMerger {
      * Copies alignment info from aligned to unaligned read, clips as appropriate, and sets PG ID.
      * May also un-map the resulting read if the alignment is bad (e.g. no unclipped bases).
      *
+     * Depending on the value of unmappingReadsStrategy this function will potentially
+     *
+     * - reset the mapping information (MOVE_TO_TAG)
+     * - copy the original mapping information to a tag "PA" (MOVE_TO_TAG, or COPY_TO_TAG)
+     * a third possibility for unmappingReadsStrategy is to do nothig (DO_NOT_CHANGE)
+     *
+     * This is because the CRAM format will reset mapping information for reads that are flagged as unaligned.
+     *
      * @param unaligned Original SAMRecord, and object into which values are copied.
      * @param aligned   Holds alignment info that will be copied into unaligned.
      * @param isContaminant Should this read be unmapped due to contamination?
@@ -557,16 +665,56 @@ public abstract class AbstractAlignmentMerger {
             log.warn("Record mapped off end of reference; making unmapped: " + aligned);
             SAMUtils.makeReadUnmapped(unaligned);
         } else if (isContaminant) {
-            log.warn("Record looks like foreign contamination; making unmapped: " + aligned);
-            // NB: for reads that look like contamination, just set unmapped flag and zero MQ but keep other flags as-is.
-            // this maintains the sort order so that downstream analyses can use them for calculating evidence
-            // of contamination vs other causes (e.g. structural variants)
+
+            crossSpeciesReads++;
+
+            if (unmappingReadsStrategy.isPopulatePaTag()) {
+                unaligned.setAttribute("PA", encodeMappingInformation(aligned));
+            }
+
+            if (unmappingReadsStrategy.isResetMappingInformation()) {
+                unaligned.setReferenceIndex(SAMRecord.NO_ALIGNMENT_REFERENCE_INDEX);
+                unaligned.setAlignmentStart(SAMRecord.NO_ALIGNMENT_START);
+                unaligned.setCigar(null);
+                unaligned.setCigarString(SAMRecord.NO_ALIGNMENT_CIGAR);
+                unaligned.setAttribute(SAMTag.NM.name(), null);
+            }
+
             unaligned.setReadUnmappedFlag(true);
+            // Unmapped read cannot have non-zero mapping quality and remain valid
             unaligned.setMappingQuality(SAMRecord.NO_MAPPING_QUALITY);
-            unaligned.setAttribute(SAMTag.FT.name(), "Cross-species contamination");
+
+            // if there already is a comment, add second comment with a | separator:
+            Optional<String> optionalComment = Optional.ofNullable(unaligned.getStringAttribute(SAMTag.CO.name()));
+            unaligned.setAttribute(optionalComment.map(s -> s + " | ").orElse("") +
+                    SAMTag.CO.name(), "Cross-species contamination");
         }
     }
 
+    /**
+     * Encodes mapping information from a record into a string according to the format sepcified in the
+     * Sam-Spec under the SA tag. No protection against missing values (for cigar, and NM tag).
+     * (Might make sense to move this to htsJDK.)
+     *
+     * @param rec SAMRecord whose alignment information will be encoded
+     * @return String encoding rec's alignment information according to SA tag in the SAM spec
+     */
+    static private String encodeMappingInformation(SAMRecord rec) {
+        return String.join(",",
+                rec.getContig(),
+                ((Integer) rec.getAlignmentStart()).toString(),
+                rec.getCigarString(),
+                ((Integer) rec.getMappingQuality()).toString(),
+                getStringOfNullable(rec.getIntegerAttribute(SAMTag.NM.name()))) + ";";
+    }
+
+    //returns the toString() of its input or an empty string if null.
+    static private String getStringOfNullable(final Object obj) {
+        return Optional.ofNullable(obj)
+                .map(Object::toString)
+                .orElse("");
+    }
+
     /**
      * Copies alignment info from aligned to unaligned read, if there is an alignment, and sets mate information.
      *
@@ -579,9 +727,15 @@ public abstract class AbstractAlignmentMerger {
     private void transferAlignmentInfoToPairedRead(final SAMRecord firstUnaligned, final SAMRecord secondUnaligned,
                                                    final SAMRecord firstAligned, final SAMRecord secondAligned,
                                                    final boolean isContaminant, final boolean needsSafeReverseComplement) {
-        if (firstAligned != null) transferAlignmentInfoToFragment(firstUnaligned, firstAligned, isContaminant, needsSafeReverseComplement);
-        if (secondAligned != null) transferAlignmentInfoToFragment(secondUnaligned, secondAligned, isContaminant, needsSafeReverseComplement);
-        if (isClipOverlappingReads()) clipForOverlappingReads(firstUnaligned, secondUnaligned);
+        if (firstAligned != null) {
+            transferAlignmentInfoToFragment(firstUnaligned, firstAligned, isContaminant, needsSafeReverseComplement);
+        }
+        if (secondAligned != null) {
+            transferAlignmentInfoToFragment(secondUnaligned, secondAligned, isContaminant, needsSafeReverseComplement);
+        }
+        if (isClipOverlappingReads()) {
+            clipForOverlappingReads(firstUnaligned, secondUnaligned);
+        }
         SamPairUtil.setMateInfo(secondUnaligned, firstUnaligned, addMateCigar);
         if (!keepAlignerProperPairFlags) {
             SamPairUtil.setProperPairFlags(secondUnaligned, firstUnaligned, expectedOrientations);
@@ -630,8 +784,12 @@ public abstract class AbstractAlignmentMerger {
         int clipped = 0;
         while (iterator.hasNext()) {
             final CigarElement elem = iterator.next();
-            if (elem.getOperator() != CigarOperator.SOFT_CLIP && elem.getOperator() != CigarOperator.HARD_CLIP) break;
-            if (elem.getOperator() == CigarOperator.SOFT_CLIP) clipped = elem.getLength();
+            if (elem.getOperator() != CigarOperator.SOFT_CLIP && elem.getOperator() != CigarOperator.HARD_CLIP) {
+                break;
+            }
+            if (elem.getOperator() == CigarOperator.SOFT_CLIP) {
+                clipped = elem.getLength();
+            }
         }
 
         return clipped;
@@ -676,11 +834,7 @@ public abstract class AbstractAlignmentMerger {
         // If it's on the negative strand, reverse complement the bases
         // and reverse the order of the qualities
         if (rec.getReadNegativeStrandFlag()) {
-            if(needsSafeReverseComplement) {
-                rec.reverseComplement();
-            } else {
-                rec.reverseComplement(true);
-            }
+            rec.reverseComplement(attributesToReverseComplement, attributesToReverse, !needsSafeReverseComplement);
         }
     }
 
@@ -722,7 +876,9 @@ public abstract class AbstractAlignmentMerger {
                     rec.getAlignmentEnd(),
                     rec.getReadLength(),
                     rec.getCigar());
-            if (null != readCigar) rec.setCigar(readCigar);
+            if (null != readCigar) {
+                rec.setCigar(readCigar);
+            }
         }
 
         // If the read's mate maps off the end of the alignment, clip it
@@ -734,7 +890,9 @@ public abstract class AbstractAlignmentMerger {
                     SAMUtils.getMateAlignmentEnd(rec), // NB: this could be computed without another call to getMateCigar
                     mateCigar.getReadLength(),
                     mateCigar);
-            if (null != mateCigar) rec.setAttribute(SAMTag.MC.name(), mateCigar.toString());
+            if (null != mateCigar) {
+                rec.setAttribute(SAMTag.MC.name(), mateCigar.toString());
+            }
         }
     }
 
@@ -760,8 +918,6 @@ public abstract class AbstractAlignmentMerger {
         }
     }
 
-    protected SAMSequenceDictionary getSequenceDictionary() { return this.sequenceDictionary; }
-
     protected SAMProgramRecord getProgramRecord() { return this.programRecord; }
 
     protected void setProgramRecord(final SAMProgramRecord pg) {
@@ -778,10 +934,14 @@ public abstract class AbstractAlignmentMerger {
 
         // All tags that start with a lower-case letter are user defined and should not be overridden by aligner
         // unless explicitly specified in attributesToRetain.
-        if (Character.isLowerCase(firstCharOfTag)) return true;
+        if (Character.isLowerCase(firstCharOfTag)) {
+            return true;
+        }
 
         for (final char c : RESERVED_ATTRIBUTE_STARTS) {
-            if (firstCharOfTag == c) return true;
+            if (firstCharOfTag == c) {
+                return true;
+            }
         }
         return false;
     }
