@@ -32,12 +32,11 @@ import htsjdk.samtools.SAMFileWriterFactory;
 import htsjdk.samtools.SAMReadGroupRecord;
 import htsjdk.samtools.SAMRecord;
 import htsjdk.samtools.SAMRecordQueryNameComparator;
-import htsjdk.samtools.SAMRecordUtil;
 import htsjdk.samtools.SAMTag;
 import htsjdk.samtools.SamReader;
 import htsjdk.samtools.SamReaderFactory;
 import htsjdk.samtools.ValidationStringency;
-import htsjdk.samtools.filter.FilteringIterator;
+import htsjdk.samtools.filter.FilteringSamIterator;
 import htsjdk.samtools.filter.SamRecordFilter;
 import htsjdk.samtools.util.CloserUtil;
 import htsjdk.samtools.util.FastqQualityFormat;
@@ -46,100 +45,150 @@ import htsjdk.samtools.util.Log;
 import htsjdk.samtools.util.PeekableIterator;
 import htsjdk.samtools.util.ProgressLogger;
 import htsjdk.samtools.util.QualityEncodingDetector;
+import htsjdk.samtools.util.RuntimeIOException;
 import htsjdk.samtools.util.SolexaQualityConverter;
 import htsjdk.samtools.util.SortingCollection;
+import org.broadinstitute.barclay.argparser.Argument;
+import org.broadinstitute.barclay.argparser.CommandLineParser;
+import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
+import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
 import picard.cmdline.CommandLineProgram;
-import picard.cmdline.CommandLineProgramProperties;
-import picard.cmdline.Option;
 import picard.cmdline.StandardOptionDefinitions;
-import picard.cmdline.programgroups.SamOrBam;
+import picard.cmdline.programgroups.ReadDataManipulationProgramGroup;
 import picard.util.TabbedTextFileWithHeaderParser;
 
 import java.io.File;
+import java.io.IOException;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
+import java.util.*;
 
 /**
  * Reverts a SAM file by optionally restoring original quality scores and by removing
  * all alignment information.
+ * <p>
+ * <p>
+ * This tool removes or restores certain properties of the SAM records, including alignment information.
+ * It can be used to produce an unmapped BAM (uBAM) from a previously aligned BAM. It is also capable of
+ * restoring the original quality scores of a BAM file that has already undergone base quality score recalibration
+ * (BQSR) if the original qualities were retained during the calibration (in the OQ tag).
+ * <p>
+ * <h3>Usage Examples</h3>
+ * <h4>Output to a single file</h4>
+ * <pre>
+ * java -jar picard.jar RevertSam \\
+ *      I=input.bam \\
+ *      O=reverted.bam
+ * </pre>
+ * <p>
+ * <h4>Output by read group into multiple files with sample map</h4>
+ * <pre>
+ * java -jar picard.jar RevertSam \\
+ *      I=input.bam \\
+ *      OUTPUT_BY_READGROUP=true \\
+ *      OUTPUT_MAP=reverted_bam_paths.tsv
+ * </pre>
+ * <p>
+ * <h4>Output by read group with no output map</h4>
+ * <pre>
+ * java -jar picard.jar RevertSam \\
+ *      I=input.bam \\
+ *      OUTPUT_BY_READGROUP=true \\
+ *      O=/write/reverted/read/group/bams/in/this/dir
+ * </pre>
+ * This will output a BAM (Can be overridden with OUTPUT_BY_READGROUP_FILE_FORMAT option.)
+ * <br/>
+ * Note: If the program fails due to a SAM validation error, consider setting the VALIDATION_STRINGENCY option to
+ * LENIENT or SILENT if the failures are expected to be obviated by the reversion process
+ * (e.g. invalid alignment information will be obviated when the REMOVE_ALIGNMENT_INFORMATION option is used).
  */
 @CommandLineProgramProperties(
-        usage = RevertSam.USAGE_SUMMARY + RevertSam.USAGE_DETAILS,
-        usageShort = RevertSam.USAGE_SUMMARY,
-        programGroup = SamOrBam.class
-)
+        summary = RevertSam.USAGE_SUMMARY + RevertSam.USAGE_DETAILS,
+        oneLineSummary = RevertSam.USAGE_SUMMARY,
+        programGroup = ReadDataManipulationProgramGroup.class)
+@DocumentedFeature
 public class RevertSam extends CommandLineProgram {
-    static final String USAGE_SUMMARY ="Reverts SAM or BAM files to a previous state.  ";
-    static final String USAGE_DETAILS ="This tool removes or restores certain properties of the SAM records, including alignment " +
+    static final String USAGE_SUMMARY = "Reverts SAM or BAM files to a previous state.  ";
+    static final String USAGE_DETAILS = "This tool removes or restores certain properties of the SAM records, including alignment " +
             "information, which can be used to produce an unmapped BAM (uBAM) from a previously aligned BAM. It is also capable of " +
             "restoring the original quality scores of a BAM file that has already undergone base quality score recalibration (BQSR) if the" +
-            "original qualities were retained." +
-            "<h4>Example with single output:</h4>" +
-            "<pre>" +
-            "java -jar picard.jar RevertSam \\<br />" +
-            "     I=input.bam \\<br />" +
-            "     O=reverted.bam" +
-            "</pre>" +
-            "Output format is BAM by default, or SAM or CRAM if the input path ends with '.sam' or '.cram', respectively." +
-            "<h4>Example outputting by read group with output map:</h4>" +
-            "<pre>" +
-            "java -jar picard.jar RevertSam \\<br />" +
-            "     I=input.bam \\<br />" +
-            "     OUTPUT_BY_READGROUP=true \\<br />" +
-            "     OUTPUT_MAP=reverted_bam_paths.tsv" +
-            "</pre>" +
-            "Will output a BAM/SAM file per read group. By default, all outputs will be in BAM format. " +
-            "However, a SAM file will be produced instead for any read group mapped in OUTPUT_MAP to a path ending with '.sam'. " +
-            "A CRAM file will be produced for any read group mapped to a path ending with '.cram'. " +
-            "<h4>Example outputting by read group without output map:</h4>" +
-            "<pre>" +
-            "java -jar picard.jar RevertSam \\<br />" +
-            "     I=input.bam \\<br />" +
-            "     OUTPUT_BY_READGROUP=true \\<br />" +
-            "     O=/write/reverted/read/group/bams/in/this/dir" +
-            "</pre>" +
-            "Will output a BAM/SAM file per read group. By default, all outputs will be in BAM format. " +
-            "However, outputs will be in SAM format if the input path ends with '.sam', or CRAM format if it ends with '.cram'." +
-            " This behaviour can be overriden with OUTPUT_BY_READGROUP_FILE_FORMAT option."+
-            "<hr />";
-    @Option(shortName = StandardOptionDefinitions.INPUT_SHORT_NAME, doc = "The input SAM/BAM file to revert the state of.")
+            "original qualities were retained.\n" +
+            "<h3>Examples</h3>\n" +
+            "<h4>Example with single output:</h4>\n" +
+            "java -jar picard.jar RevertSam \\\n" +
+            "     I=input.bam \\\n" +
+            "     O=reverted.bam\n" +
+            "\n" +
+            "<h4>Example outputting by read group with output map:</h4>\n" +
+            "java -jar picard.jar RevertSam \\\n" +
+            "     I=input.bam \\\n" +
+            "     OUTPUT_BY_READGROUP=true \\\n" +
+            "     OUTPUT_MAP=reverted_bam_paths.tsv\n" +
+            "\n" +
+            "Will output a BAM/SAM file per read group.\n" +
+            "<h4>Example outputting by read group without output map:</h4>\n" +
+            "java -jar picard.jar RevertSam \\\n" +
+            "     I=input.bam \\\n" +
+            "     OUTPUT_BY_READGROUP=true \\\n" +
+            "     O=/write/reverted/read/group/bams/in/this/dir\n" +
+            "\n" +
+            "Will output a BAM file per read group." +
+            " Output format can be overridden with the OUTPUT_BY_READGROUP_FILE_FORMAT option.\n" +
+            "Note: If the program fails due to a SAM validation error, consider setting the VALIDATION_STRINGENCY option to " +
+            "LENIENT or SILENT if the failures are expected to be obviated by the reversion process " +
+            "(e.g. invalid alignment information will be obviated when the REMOVE_ALIGNMENT_INFORMATION option is used).\n" +
+            "";
+    @Argument(shortName = StandardOptionDefinitions.INPUT_SHORT_NAME, doc = "The input SAM/BAM file to revert the state of.")
     public File INPUT;
 
-    @Option(mutex = {"OUTPUT_MAP"}, shortName = StandardOptionDefinitions.OUTPUT_SHORT_NAME, doc = "The output SAM/BAM file to create, or an output directory if OUTPUT_BY_READGROUP is true.")
+    @Argument(mutex = {"OUTPUT_MAP"}, shortName = StandardOptionDefinitions.OUTPUT_SHORT_NAME, doc = "The output SAM/BAM file to create, or an output directory if OUTPUT_BY_READGROUP is true.")
     public File OUTPUT;
-    
-    @Option(mutex = {"OUTPUT"}, shortName = "OM", doc = "Tab separated file with two columns, READ_GROUP_ID and OUTPUT, providing file mapping only used if OUTPUT_BY_READGROUP is true.")
+
+    @Argument(mutex = {"OUTPUT"}, shortName = "OM", doc = "Tab separated file with two columns, READ_GROUP_ID and OUTPUT, providing file mapping only used if OUTPUT_BY_READGROUP is true.")
     public File OUTPUT_MAP;
 
-    @Option(shortName = "OBR", doc = "When true, outputs each read group in a separate file.")
+    @Argument(shortName = "OBR", doc = "When true, outputs each read group in a separate file.")
     public boolean OUTPUT_BY_READGROUP = false;
 
-    public static enum FileType {sam, bam, cram,dynamic}
-    @Option(shortName = "OBRFF", doc = "When using OUTPUT_BY_READGROUP, the output file format can be set to a certain format." )
-    public FileType  OUTPUT_BY_READGROUP_FILE_FORMAT=FileType.dynamic;
+    public static enum FileType implements CommandLineParser.ClpEnum {
+        sam("Generate SAM files."),
+        bam("Generate BAM files."),
+        cram("Generate CRAM files."),
+        dynamic("Generate files based on the extention of INPUT.");
 
-    @Option(shortName = "SO", doc = "The sort order to create the reverted output file with.")
+        final String description;
+
+        FileType(String descrition) {
+            this.description = descrition;
+        }
+
+        @Override
+        public String getHelpDoc() {
+            return description;
+        }
+    }
+
+    @Argument(shortName = "OBRFF", doc = "When using OUTPUT_BY_READGROUP, the output file format can be set to a certain format.")
+    public FileType OUTPUT_BY_READGROUP_FILE_FORMAT = FileType.dynamic;
+
+    @Argument(shortName = "SO", doc = "The sort order to create the reverted output file with.")
     public SortOrder SORT_ORDER = SortOrder.queryname;
 
-    @Option(shortName = StandardOptionDefinitions.USE_ORIGINAL_QUALITIES_SHORT_NAME, doc = "True to restore original qualities from the OQ field to the QUAL field if available.")
+    @Argument(shortName = StandardOptionDefinitions.USE_ORIGINAL_QUALITIES_SHORT_NAME, doc = "True to restore original qualities from the OQ field to the QUAL field if available.")
     public boolean RESTORE_ORIGINAL_QUALITIES = true;
 
-    @Option(doc = "Remove duplicate read flags from all reads.  Note that if this is true and REMOVE_ALIGNMENT_INFORMATION==false, " +
+    @Argument(doc = "Remove duplicate read flags from all reads.  Note that if this is false and REMOVE_ALIGNMENT_INFORMATION==true, " +
             " the output may have the unusual but sometimes desirable trait of having unmapped reads that are marked as duplicates.")
     public boolean REMOVE_DUPLICATE_INFORMATION = true;
 
-    @Option(doc = "Remove all alignment information from the file.")
+    @Argument(doc = "Remove all alignment information from the file.")
     public boolean REMOVE_ALIGNMENT_INFORMATION = true;
 
-    @Option(doc = "When removing alignment information, the set of optional tags to remove.")
+    @Argument(doc = "When removing alignment information, the set of optional tags to remove.")
     public List<String> ATTRIBUTE_TO_CLEAR = new ArrayList<String>() {{
         add(SAMTag.NM.name());
         add(SAMTag.UQ.name());
@@ -147,37 +196,37 @@ public class RevertSam extends CommandLineProgram {
         add(SAMTag.MD.name());
         add(SAMTag.MQ.name());
         add(SAMTag.SA.name()); // Supplementary alignment metadata
-        add(SAMTag.MC.name());      // Mate Cigar
+        add(SAMTag.MC.name()); // Mate Cigar
         add(SAMTag.AS.name());
     }};
 
-    @Option(doc = "WARNING: This option is potentially destructive. If enabled will discard reads in order to produce " +
+    @Argument(doc = "WARNING: This option is potentially destructive. If enabled will discard reads in order to produce " +
             "a consistent output BAM. Reads discarded include (but are not limited to) paired reads with missing " +
             "mates, duplicated records, records with mismatches in length of bases and qualities. This option can " +
             "only be enabled if the output sort order is queryname and will always cause sorting to occur.")
     public boolean SANITIZE = false;
 
-    @Option(doc = "If SANITIZE=true and higher than MAX_DISCARD_FRACTION reads are discarded due to sanitization then" +
+    @Argument(doc = "If SANITIZE=true and higher than MAX_DISCARD_FRACTION reads are discarded due to sanitization then" +
             "the program will exit with an Exception instead of exiting cleanly. Output BAM will still be valid.")
     public double MAX_DISCARD_FRACTION = 0.01;
 
-    @Option(doc = "The sample alias to use in the reverted output file.  This will override the existing " +
+    @Argument(doc = "If SANITIZE=true keep the first record when we find more than one record with the same name for R1/R2/unpaired reads respectively. " +
+            "For paired end reads, keeps only the first R1 and R2 found respectively, and discards all unpaired reads. " +
+            "Duplicates do not refer to the duplicate flag in the FLAG field, but instead reads with the same name.")
+    public boolean KEEP_FIRST_DUPLICATE = false;
+
+    @Argument(doc = "The sample alias to use in the reverted output file.  This will override the existing " +
             "sample alias in the file and is used only if all the read groups in the input file have the " +
-            "same sample alias ", shortName = StandardOptionDefinitions.SAMPLE_ALIAS_SHORT_NAME, optional = true)
+            "same sample alias.", shortName = StandardOptionDefinitions.SAMPLE_ALIAS_SHORT_NAME, optional = true)
     public String SAMPLE_ALIAS;
 
-    @Option(doc = "The library name to use in the reverted output file.  This will override the existing " +
+    @Argument(doc = "The library name to use in the reverted output file.  This will override the existing " +
             "sample alias in the file and is used only if all the read groups in the input file have the " +
-            "same library name ", shortName = StandardOptionDefinitions.LIBRARY_NAME_SHORT_NAME, optional = true)
+            "same library name.", shortName = StandardOptionDefinitions.LIBRARY_NAME_SHORT_NAME, optional = true)
     public String LIBRARY_NAME;
 
     private final static Log log = Log.getInstance(RevertSam.class);
 
-    /** Default main method impl. */
-    public static void main(final String[] args) {
-        new RevertSam().instanceMainWithExit(args);
-    }
-
     /**
      * Enforce that output ordering is queryname when sanitization is turned on since it requires a queryname sort.
      */
@@ -187,6 +236,8 @@ public class RevertSam extends CommandLineProgram {
         ValidationUtil.validateSanitizeSortOrder(SANITIZE, SORT_ORDER, errors);
         ValidationUtil.validateOutputParams(OUTPUT_BY_READGROUP, OUTPUT, OUTPUT_MAP, errors);
 
+        if (!SANITIZE && KEEP_FIRST_DUPLICATE) errors.add("KEEP_FIRST_DUPLICATE cannot be used without SANITIZE");
+
         if (!errors.isEmpty()) {
             return errors.toArray(new String[errors.size()]);
         }
@@ -214,9 +265,12 @@ public class RevertSam extends CommandLineProgram {
         final Map<String, File> outputMap;
         final Map<String, SAMFileHeader> headerMap;
         if (OUTPUT_BY_READGROUP) {
+            if (inHeader.getReadGroups().isEmpty()) {
+                throw new PicardException(INPUT + " does not contain Read Groups");
+            }
 
             final String defaultExtension;
-            if (OUTPUT_BY_READGROUP_FILE_FORMAT==FileType.dynamic) {
+            if (OUTPUT_BY_READGROUP_FILE_FORMAT == FileType.dynamic) {
                 defaultExtension = getDefaultExtension(INPUT.toString());
             } else {
                 defaultExtension = "." + OUTPUT_BY_READGROUP_FILE_FORMAT.toString();
@@ -232,12 +286,13 @@ public class RevertSam extends CommandLineProgram {
 
         final SAMFileWriterFactory factory = new SAMFileWriterFactory();
         final RevertSamWriter out = new RevertSamWriter(OUTPUT_BY_READGROUP, headerMap, outputMap, singleOutHeader, OUTPUT, presorted, factory, REFERENCE_SEQUENCE);
-        
+
         ////////////////////////////////////////////////////////////////////////////
         // Build a sorting collection to use if we are sanitizing
         ////////////////////////////////////////////////////////////////////////////
         final RevertSamSorter sorter;
-        if (sanitizing) sorter = new RevertSamSorter(OUTPUT_BY_READGROUP, headerMap, singleOutHeader, MAX_RECORDS_IN_RAM);
+        if (sanitizing)
+            sorter = new RevertSamSorter(OUTPUT_BY_READGROUP, headerMap, singleOutHeader, MAX_RECORDS_IN_RAM);
         else sorter = null;
 
         final ProgressLogger progress = new ProgressLogger(log, 1000000, "Reverted");
@@ -254,6 +309,7 @@ public class RevertSam extends CommandLineProgram {
             if (sanitizing) sorter.add(rec);
             else out.addAlignment(rec);
         }
+        CloserUtil.close(in);
 
         ////////////////////////////////////////////////////////////////////////////
         // Now if we're sanitizing, clean up the records and write them to the output
@@ -283,7 +339,6 @@ public class RevertSam extends CommandLineProgram {
             }
         }
 
-        CloserUtil.close(in);
         return 0;
     }
 
@@ -356,36 +411,65 @@ public class RevertSam extends CommandLineProgram {
         for (final PeekableIterator<SAMRecord> iterator : iterators) {
             readNameLoop:
             while (iterator.hasNext()) {
-                final List<SAMRecord> recs = fetchByReadName(iterator);
+                List<SAMRecord> recs = fetchByReadName(iterator);
                 total += recs.size();
 
                 // Check that all the reads have bases and qualities of the same length
                 for (final SAMRecord rec : recs) {
                     if (rec.getReadBases().length != rec.getBaseQualities().length) {
-                        log.debug("Discarding " + recs.size() + " reads with name " + rec.getReadName() + " for mismatching bases and quals length.");
+                        log.debug("Discarding ", recs.size(), " reads with name ", rec.getReadName(), " for mismatching bases and quals length.");
                         discarded += recs.size();
                         continue readNameLoop;
                     }
                 }
 
-                // Check that if the first read is marked as unpaired that there is in fact only one read
-                if (!recs.get(0).getReadPairedFlag() && recs.size() > 1) {
-                    log.debug("Discarding " + recs.size() + " reads with name " + recs.get(0).getReadName() + " because they claim to be unpaired.");
-                    discarded += recs.size();
-                    continue readNameLoop;
+                // Get the number of R1s, R2s, and unpaired reads respectively.
+                int firsts = 0, seconds = 0, unpaired = 0;
+                SAMRecord firstRecord = null, secondRecord = null, unpairedRecord = null;
+                for (final SAMRecord rec : recs) {
+                    if (!rec.getReadPairedFlag()) {
+                        if (unpairedRecord == null) {
+                            unpairedRecord = rec;
+                        }
+                        ++unpaired;
+                    } else {
+                        if (rec.getFirstOfPairFlag()) {
+                            if (firstRecord == null) {
+                                firstRecord = rec;
+                            }
+                            ++firsts;
+                        }
+                        if (rec.getSecondOfPairFlag()) {
+                            if (secondRecord == null) {
+                                secondRecord = rec;
+                            }
+                            ++seconds;
+                        }
+                    }
                 }
 
-                // Check that if we have paired reads there is exactly one first of pair and one second of pair
-                if (recs.get(0).getReadPairedFlag()) {
-                    int firsts = 0, seconds = 0, unpaired = 0;
-                    for (final SAMRecord rec : recs) {
-                        if (!rec.getReadPairedFlag()) ++unpaired;
-                        if (rec.getFirstOfPairFlag()) ++firsts;
-                        if (rec.getSecondOfPairFlag()) ++seconds;
-                    }
+                // If we have paired reads, then check that there is exactly one first of pair and one second of pair.
+                // Otherwise, check that we have only one unpaired read.
+                if (firsts > 0 || seconds > 0) { // if we have any paired reads
+                    if (firsts != 1 || seconds != 1) { // if we do not have exactly one R1 and one R2
+                        if (KEEP_FIRST_DUPLICATE && firsts >= 1 && seconds >= 1) { // if we have at least one R1 and one R2, we can discard all but the first encountered
+                            discarded += recs.size() - 2;
+                            recs = Arrays.asList(firstRecord, secondRecord);
+                        }  else {
+                            log.debug("Discarding ", recs.size(), " reads with name ", recs.get(0).getReadName(), " because  we found ", firsts, " R1s ", seconds, " R2s and ", unpaired, " unpaired reads.");
+                            discarded += recs.size();
+                            continue readNameLoop;
+                        }
 
-                    if (unpaired > 0 || firsts != 1 || seconds != 1) {
-                        log.debug("Discarding " + recs.size() + " reads with name " + recs.get(0).getReadName() + " because pairing information in corrupt.");
+                    }
+                }
+                else if (unpaired > 1) { // only unpaired reads, and we have too many
+                    if (KEEP_FIRST_DUPLICATE) {
+                        discarded += recs.size() - 1;
+                        recs = Collections.singletonList(unpairedRecord);
+                    }
+                    else {
+                        log.debug("Discarding ", recs.size(), " reads with name ", recs.get(0).getReadName(), " because we found ", unpaired, " unpaired reads.");
                         discarded += recs.size();
                         continue readNameLoop;
                     }
@@ -395,7 +479,7 @@ public class RevertSam extends CommandLineProgram {
                 for (final SAMRecord rec : recs) {
                     // The only valid quality score encoding scheme is standard; if it's not standard, change it.
                     final FastqQualityFormat recordFormat = readGroupToFormat.get(rec.getReadGroup());
-                    if (!recordFormat.equals(FastqQualityFormat.Standard)) {
+                    if (recordFormat != null && !recordFormat.equals(FastqQualityFormat.Standard)) {
                         final byte[] quals = rec.getBaseQualities();
                         for (int i = 0; i < quals.length; i++) {
                             quals[i] -= SolexaQualityConverter.ILLUMINA_TO_PHRED_SUBTRAHEND;
@@ -409,7 +493,7 @@ public class RevertSam extends CommandLineProgram {
         }
         return new long[]{discarded, total};
     }
-    
+
     /**
      * Generates a list by consuming from the iterator in order starting with the first available
      * read and continuing while subsequent reads share the same read name. If there are no reads
@@ -481,7 +565,7 @@ public class RevertSam extends CommandLineProgram {
             final SAMFileHeader inHeader,
             final SortOrder sortOrder,
             final boolean removeAlignmentInformation) {
-        
+
         final Map<String, SAMFileHeader> headerMap = new HashMap<>();
         for (final SAMReadGroupRecord readGroup : inHeader.getReadGroups()) {
             final SAMFileHeader header = createOutHeader(inHeader, sortOrder, removeAlignmentInformation);
@@ -514,9 +598,10 @@ public class RevertSam extends CommandLineProgram {
 
         final Map<SAMReadGroupRecord, FastqQualityFormat> readGroupToFormat = new HashMap<>();
 
+        final SamReaderFactory readerFactory =
+                SamReaderFactory.makeDefault().referenceSequence(referenceSequence).validationStringency(validationStringency);
         // Figure out the quality score encoding scheme for each read group.
         for (final SAMReadGroupRecord rg : inHeader.getReadGroups()) {
-            final SamReader reader = SamReaderFactory.makeDefault().referenceSequence(referenceSequence).validationStringency(validationStringency).open(input);
             final SamRecordFilter filter = new SamRecordFilter() {
                 public boolean filterOut(final SAMRecord rec) {
                     return !rec.getReadGroup().getId().equals(rg.getId());
@@ -526,9 +611,19 @@ public class RevertSam extends CommandLineProgram {
                     throw new UnsupportedOperationException();
                 }
             };
-            readGroupToFormat.put(rg, QualityEncodingDetector.detect(QualityEncodingDetector.DEFAULT_MAX_RECORDS_TO_ITERATE, new FilteringIterator(reader.iterator(), filter), restoreOriginalQualities));
-            CloserUtil.close(reader);
+            try (final SamReader reader = readerFactory.open(input)) {
+                final FilteringSamIterator filterIterator = new FilteringSamIterator(reader.iterator(), filter);
+                if (filterIterator.hasNext()) {
+                    readGroupToFormat.put(rg, QualityEncodingDetector.detect(
+                            QualityEncodingDetector.DEFAULT_MAX_RECORDS_TO_ITERATE, filterIterator, restoreOriginalQualities));
+                } else {
+                    log.warn("Skipping read group " + rg.getReadGroupId() + " with no reads");
+                }
+            } catch (IOException e) {
+                throw new RuntimeIOException(e);
+            }
         }
+
         for (final SAMReadGroupRecord r : readGroupToFormat.keySet()) {
             log.info("Detected quality format for " + r.getReadGroupId() + ": " + readGroupToFormat.get(r));
         }
@@ -753,16 +848,9 @@ public class RevertSam extends CommandLineProgram {
         }
 
         static boolean isOutputMapHeaderValid(final List<String> columnLabels) {
-            if (columnLabels.size() < 2) {
-                return false;
-            }
-            if (!"READ_GROUP_ID".equals(columnLabels.get(0))) {
-                return false;
-            }
-            if (!"OUTPUT".equals(columnLabels.get(1))) {
-                return false;
-            }
-            return true;
+            return columnLabels.size() >= 2 &&
+                    "READ_GROUP_ID".equals(columnLabels.get(0)) &&
+                    "OUTPUT".equals(columnLabels.get(1));
         }
     }
 }
