@@ -9,27 +9,27 @@ import htsjdk.samtools.SAMRecordSetBuilder;
 import htsjdk.samtools.SAMTextHeaderCodec;
 import htsjdk.samtools.metrics.MetricsFile;
 import htsjdk.samtools.util.BufferedLineReader;
+import org.apache.commons.lang3.StringUtils;
 import org.testng.Assert;
 import org.testng.annotations.BeforeTest;
 import org.testng.annotations.Test;
 import picard.cmdline.CommandLineProgramTest;
 import picard.sam.SortSam;
 import static picard.analysis.GcBiasMetricsCollector.PerUnitGcBiasMetricsCollector.*;
-
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.FileReader;
 import java.io.IOException;
+import java.util.HashSet;
 import java.util.Random;
+import java.util.Set;
 
 /**
  * Tests the two default "programs" that have tests in CollectMultipleMetrics
  *
- *
  * @author Yossi farjoun
  */
-
 public class CollectMultipleMetricsTest extends CommandLineProgramTest {
 
     private static final File TEST_DATA_DIR = new File("testdata/picard/sam");
@@ -38,21 +38,20 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
         return CollectMultipleMetrics.class.getSimpleName();
     }
 
-
     @Test
     public void testAlignmentSummaryViaMultipleMetrics() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile   = File.createTempFile("alignmentMetrics", "");
+        final File outfile = File.createTempFile("alignmentMetrics", "");
         outfile.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
-                "METRIC_ACCUMULATION_LEVEL="+MetricAccumulationLevel.ALL_READS.name(),
+                "METRIC_ACCUMULATION_LEVEL=" + MetricAccumulationLevel.ALL_READS.name(),
                 "PROGRAM=null",
-                "PROGRAM="+CollectMultipleMetrics.Program.CollectAlignmentSummaryMetrics.name(),
-                "PROGRAM="+CollectMultipleMetrics.Program.CollectInsertSizeMetrics.name()
+                "PROGRAM=" + CollectMultipleMetrics.Program.CollectAlignmentSummaryMetrics.name(),
+                "PROGRAM=" + CollectMultipleMetrics.Program.CollectInsertSizeMetrics.name()
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
@@ -105,29 +104,27 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
     @Test
     public void testInsertSize() throws IOException {
         final File input = new File(TEST_DATA_DIR, "insert_size_metrics_test.sam");
-        final File outfile   = File.createTempFile("test", "");
+        final File outfile = File.createTempFile("test", "");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File pdf   = File.createTempFile("test", ".pdf");
+        final File pdf = File.createTempFile("test", ".pdf");
         outfile.deleteOnExit();
         pdf.deleteOnExit();
-        final String[] args = new String[] {
-                "INPUT="  + input.getAbsolutePath(),
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + reference.getAbsolutePath(),
-                "METRIC_ACCUMULATION_LEVEL="+MetricAccumulationLevel.ALL_READS.name(),
+                "METRIC_ACCUMULATION_LEVEL=" + MetricAccumulationLevel.ALL_READS.name(),
                 "PROGRAM=null",
-                "PROGRAM="+CollectMultipleMetrics.Program.CollectAlignmentSummaryMetrics.name(),
-                "PROGRAM="+CollectMultipleMetrics.Program.CollectInsertSizeMetrics.name()
+                "PROGRAM=" + CollectMultipleMetrics.Program.CollectAlignmentSummaryMetrics.name(),
+                "PROGRAM=" + CollectMultipleMetrics.Program.CollectInsertSizeMetrics.name()
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
-
         final MetricsFile<InsertSizeMetrics, Comparable<?>> output = new MetricsFile<InsertSizeMetrics, Comparable<?>>();
         output.read(new FileReader(outfile + ".insert_size_metrics"));
-
         for (final InsertSizeMetrics metrics : output.getMetrics()) {
             Assert.assertEquals(metrics.PAIR_ORIENTATION.name(), "FR");
-            if (metrics.LIBRARY==null) {  // SAMPLE or ALL_READS level
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 41);
+            if (metrics.LIBRARY == null) {  // SAMPLE or ALL_READS level
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 41);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 45);
                 Assert.assertEquals(metrics.READ_PAIRS, 13);
@@ -142,9 +139,8 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 11);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 11);
 
-            }
-            else if (metrics.LIBRARY.equals("Solexa-41753")) { // one LIBRARY and one READ_GROUP
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 44);
+            } else if (metrics.LIBRARY.equals("Solexa-41753")) { // one LIBRARY and one READ_GROUP
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 44);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 44);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 44);
                 Assert.assertEquals(metrics.READ_PAIRS, 2);
@@ -159,9 +155,8 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 1);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 1);
 
-            }
-            else if (metrics.LIBRARY.equals("Solexa-41748") && metrics.READ_GROUP == null) {
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 40);
+            } else if (metrics.LIBRARY.equals("Solexa-41748") && metrics.READ_GROUP == null) {
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 40);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 45);
                 Assert.assertEquals(metrics.READ_PAIRS, 9);
@@ -176,9 +171,8 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 11);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 11);
 
-            }
-            else if (metrics.LIBRARY.equals("Solexa-41734") && metrics.READ_GROUP == null) {
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 26);
+            } else if (metrics.LIBRARY.equals("Solexa-41734") && metrics.READ_GROUP == null) {
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 26);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 41);
                 Assert.assertEquals(metrics.READ_PAIRS, 9);
@@ -192,9 +186,8 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_80_PERCENT, 11);
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 11);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 11);
-            }
-            else if (metrics.READ_GROUP.equals("62A79AAXX100907.7")) {
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 36);
+            } else if (metrics.READ_GROUP.equals("62A79AAXX100907.7")) {
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 41);
                 Assert.assertEquals(metrics.READ_PAIRS, 4);
@@ -208,9 +201,8 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_80_PERCENT, 11);
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 11);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 11);
-            }
-            else if (metrics.READ_GROUP.equals("62A79AAXX100907.6")) {
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 41);
+            } else if (metrics.READ_GROUP.equals("62A79AAXX100907.6")) {
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 41);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 38);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 45);
                 Assert.assertEquals(metrics.READ_PAIRS, 5);
@@ -224,9 +216,8 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_80_PERCENT, 7);
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 9);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 9);
-            }
-            else if (metrics.READ_GROUP.equals("62A79AAXX100907.5")) {
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 41);
+            } else if (metrics.READ_GROUP.equals("62A79AAXX100907.5")) {
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 41);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 41);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 41);
                 Assert.assertEquals(metrics.READ_PAIRS, 1);
@@ -240,9 +231,8 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_80_PERCENT, 1);
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 1);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 1);
-            }
-            else if (metrics.READ_GROUP.equals("62A79AAXX100907.3")) {
-                Assert.assertEquals((int)metrics.MEDIAN_INSERT_SIZE, 36);
+            } else if (metrics.READ_GROUP.equals("62A79AAXX100907.3")) {
+                Assert.assertEquals((int) metrics.MEDIAN_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.MIN_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.MAX_INSERT_SIZE, 36);
                 Assert.assertEquals(metrics.READ_PAIRS, 1);
@@ -256,15 +246,55 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 Assert.assertEquals(metrics.WIDTH_OF_80_PERCENT, 1);
                 Assert.assertEquals(metrics.WIDTH_OF_90_PERCENT, 1);
                 Assert.assertEquals(metrics.WIDTH_OF_99_PERCENT, 1);
-            }
-            else {
+            } else {
                 Assert.fail("Unexpected metric: " + metrics);
             }
         }
     }
 
+    @Test
+    public void testRnaSeqMetricsViaMultipleMetrics() throws Exception {
+        final File input = new File(TEST_DATA_DIR, "rna_seq_metrics.sam");
+        final File outfile = File.createTempFile("tmp.", ".rna_metrics");
+        final File intervals = new File(TEST_DATA_DIR, "rna_seq_metrics.interval_list");
+        final File refflat = new File(TEST_DATA_DIR, "rna_seq_metrics.refflat");
+        final Set<String> ignoreSequence = new HashSet<>();
+        ignoreSequence.add("chrM");
+        outfile.deleteOnExit();
+        final String[] args = new String[]{
+                "INPUT=" + input.getAbsolutePath(),
+                "OUTPUT=" + outfile.getAbsolutePath(),
+                "METRIC_ACCUMULATION_LEVEL=" + MetricAccumulationLevel.ALL_READS.name(),
+                "INTERVALS=" + intervals.getAbsolutePath(),
+                "REF_FLAT=" + refflat.getAbsolutePath(),
+                "IGNORE_SEQUENCE=" + StringUtils.join(ignoreSequence, ','),
+                "PROGRAM=" + CollectMultipleMetrics.Program.RnaSeqMetrics.name(),
+        };
+        Assert.assertEquals(runPicardCommandLine(args), 0);
+
+        final MetricsFile<RnaSeqMetrics, Comparable<?>> output = new MetricsFile<RnaSeqMetrics, Comparable<?>>();
+        output.read(new FileReader(outfile + ".rna_metrics"));
+        final RnaSeqMetrics metrics = output.getMetrics().get(0);
+
+        Assert.assertEquals(metrics.PF_ALIGNED_BASES, 396);
+        Assert.assertEquals(metrics.PF_BASES, 432);
+        Assert.assertEquals(metrics.RIBOSOMAL_BASES.longValue(), 108L);
+        Assert.assertEquals(metrics.CODING_BASES, 136);
+        Assert.assertEquals(metrics.UTR_BASES, 51);
+        Assert.assertEquals(metrics.INTRONIC_BASES, 50);
+        Assert.assertEquals(metrics.INTERGENIC_BASES, 51);
+        Assert.assertEquals(metrics.CORRECT_STRAND_READS, 3);
+        Assert.assertEquals(metrics.INCORRECT_STRAND_READS, 4);
+        Assert.assertEquals(metrics.IGNORED_READS, 1);
+        Assert.assertEquals(metrics.NUM_R1_TRANSCRIPT_STRAND_READS, 1);
+        Assert.assertEquals(metrics.NUM_R2_TRANSCRIPT_STRAND_READS, 2);
+        Assert.assertEquals(metrics.NUM_UNEXPLAINED_READS, 2);
+        Assert.assertEquals(metrics.PCT_R1_TRANSCRIPT_STRAND_READS, 0.333333);
+        Assert.assertEquals(metrics.PCT_R2_TRANSCRIPT_STRAND_READS, 0.666667);
+    }
+
     @Test //test all gcBias collection levels
-    public void testGcBiasMetrics() throws IOException{
+    public void testGcBiasMetrics() throws IOException {
         runGcTest(tempSamFile);
     }
 
@@ -276,11 +306,11 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
                 "REFERENCE_SEQUENCE=" + referenceFile,
-                "METRIC_ACCUMULATION_LEVEL="+MetricAccumulationLevel.ALL_READS.name(),
+                "METRIC_ACCUMULATION_LEVEL=" + MetricAccumulationLevel.ALL_READS.name(),
                 "PROGRAM=null",
-                "PROGRAM="+CollectMultipleMetrics.Program.CollectAlignmentSummaryMetrics.name(),
-                "PROGRAM="+CollectMultipleMetrics.Program.CollectInsertSizeMetrics.name(),
-                "PROGRAM="+CollectMultipleMetrics.Program.CollectGcBiasMetrics.name()
+                "PROGRAM=" + CollectMultipleMetrics.Program.CollectAlignmentSummaryMetrics.name(),
+                "PROGRAM=" + CollectMultipleMetrics.Program.CollectInsertSizeMetrics.name(),
+                "PROGRAM=" + CollectMultipleMetrics.Program.CollectGcBiasMetrics.name()
         };
         Assert.assertEquals(runPicardCommandLine(args), 0);
 
@@ -315,15 +345,16 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
     private final static String library1 = "TestLibrary1";
     private final static String library2 = "TestLibrary2";
     private final static String library3 = "TestLibrary3";
-
     private final static File TEST_DIR = new File("testdata/picard/sam/CollectGcBiasMetrics/");
     private final File dict = new File("testdata/picard/quality/chrM.reference.dict");
-
     private File tempSamFile;
+    private final SAMRecordSetBuilder setBuilder1 =
+            new SAMRecordSetBuilder(true, SAMFileHeader.SortOrder.coordinate);
+    private final SAMRecordSetBuilder setBuilder2 =
+            new SAMRecordSetBuilder(true, SAMFileHeader.SortOrder.coordinate);
+    private final SAMRecordSetBuilder setBuilder3 =
+            new SAMRecordSetBuilder(true, SAMFileHeader.SortOrder.coordinate);
 
-    private final SAMRecordSetBuilder setBuilder1 = new SAMRecordSetBuilder(true, SAMFileHeader.SortOrder.coordinate);
-    private final SAMRecordSetBuilder setBuilder2 = new SAMRecordSetBuilder(true, SAMFileHeader.SortOrder.coordinate);
-    private final SAMRecordSetBuilder setBuilder3 = new SAMRecordSetBuilder(true, SAMFileHeader.SortOrder.coordinate);
     private final SAMReadGroupRecord readGroupRecord1 = new SAMReadGroupRecord(readGroupId1);
     private final SAMReadGroupRecord readGroupRecord2 = new SAMReadGroupRecord(readGroupId2);
     private final SAMReadGroupRecord readGroupRecord3 = new SAMReadGroupRecord(readGroupId3);
@@ -333,23 +364,21 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
     void setupBuilder() throws IOException {
         final int numReads = 100;
         final String flowCellBarcode = "TESTBARCODE";
-
         tempSamFile = File.createTempFile("CollectGcBias", ".bam", TEST_DIR);
+        final File tempSamIndex = new File(tempSamFile.getAbsolutePath().replace("bam", "bai"));
         final File tempSamFileUnsorted = File.createTempFile("CollectGcBias", ".bam", TEST_DIR);
         tempSamFileUnsorted.deleteOnExit();
+        tempSamIndex.deleteOnExit();
         tempSamFile.deleteOnExit();
-
         BufferedLineReader bufferedLineReader = null;
         try {
             bufferedLineReader = new BufferedLineReader(new FileInputStream(dict));
         } catch (FileNotFoundException e) {
             e.printStackTrace();
         }
-
         final SAMTextHeaderCodec codec = new SAMTextHeaderCodec();
         final SAMFileHeader header = codec.decode(bufferedLineReader, dict.toString());
         header.setSortOrder(SAMFileHeader.SortOrder.unsorted);
-
         //build different levels to put into the same bam file for testing multi level collection
         setup(numReads, flowCellBarcode, 1, readGroupId1, readGroupRecord1, sample1, library1, header, setBuilder1); //Sample 1, Library 1, RG 1
         setup(numReads, flowCellBarcode, 2, readGroupId2, readGroupRecord2, sample1, library2, header, setBuilder2); //Sample 1, Library 2, RG 2
@@ -371,10 +400,12 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
 
         //sort the temp file
         final SortSam sorter = new SortSam();
-        final String[] args = new String[]{"INPUT=" + tempSamFileUnsorted.getAbsolutePath(), "OUTPUT=" + tempSamFile.getAbsolutePath(), "SORT_ORDER=coordinate"};
-
+        final String[] args = new String[]{"INPUT=" + tempSamFileUnsorted.getAbsolutePath(),
+                "OUTPUT=" + tempSamFile.getAbsolutePath(),
+                "SORT_ORDER=coordinate"};
         sorter.instanceMain(args);
     }
+
     void setup(final int numReads,
                final String readName,
                final int ID,
@@ -384,7 +415,6 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
                final String library,
                final SAMFileHeader header,
                final SAMRecordSetBuilder setBuilder) throws IOException {
-
         final String separator = ":";
         readGroupRecord.setSample(sample);
         readGroupRecord.setPlatform(platform);
@@ -393,7 +423,6 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
         header.addReadGroup(readGroupRecord);
         setBuilder.setReadGroup(readGroupRecord);
         setBuilder.setUseNmFlag(true);
-
         setBuilder.setHeader(header);
 
         final int max = 15000;
@@ -403,7 +432,7 @@ public class CollectMultipleMetricsTest extends CommandLineProgramTest {
         for (int i = 0; i < numReads; i++) {
             final int start = rg.nextInt(max) + min;
             final String newReadName = readName + separator + ID + separator + i;
-            setBuilder.addPair(newReadName, 0, start+ID, start+ID+99);
+            setBuilder.addPair(newReadName, 0, start + ID, start + ID + 99);
         }
     }
 
